"""
Generates a series of keys and make sures the function is taking 
in arguments correctly.
"""
import seash
import os.path
import sys

orig_stdout = sys.stdout
sys.stdout = open("test_results.txt", "w")

command_list = [
  "genkeys joe", 
  "genkeys bob 50", 
  "genkeys sam as john", 
  "loadkeys sam", 
  "as sam", 
  "loadkeys sam", 
  "as john"
  ]

seash.command_loop(command_list)

sys.stdout.close()

sys.stdout = orig_stdout


# Compares the output with the expected output in the provided txt file.
# Results should hold a few simple lines such as "Identity bob has been created"
test_results = open("test_results.txt", "r")
wanted_results = open("genkeys_test_results.txt", "r")

original = wanted_results.readlines()
actual = test_results.readlines()

for i in range(len(original)):
  if not original[i].startswith(actual[i]):
    print "Line " + str(i) + " of test results are not consistent with expected results: genkeys_test_results.txt"

test_results.close()
wanted_results.close()



# Check that the created keys total character count is at least less than 60
testpublickey = open('bob.publickey')
testprivatekey = open('bob.privatekey')

publickey = testpublickey.readline()
privatekey = testprivatekey.readline()

publickeylist = publickey.split()
if not len(publickeylist[1]) + len(privatekey) < 60:
  print "Length of generated key 'bob' is far greater than expected"

os.remove("joe.publickey")
os.remove("joe.privatekey")




"""
Uploads and runs a simple repy file to make sure no errors gets thrown, and
proceeds to download and remove the file from the node to test seash's file
recognition.
"""
#pragma out The specified file(s) could not be found. Please check the filename.

import seash
import sys


# Prevent printing to console by using test_results.txt as a text dump and redirecting output there
orig_stdout = sys.stdout
sys.stdout = open("test_results.txt", "w")


command_list = [
  'loadkeys guest0', 
  'as guest0', 
  'browse', 
  'on %1', 
  'loadkeys guest0', 
  'as guest0', 
  'upload example.1.1.r2py sample_file', 
  'run example.1.1.r2py test_argument', 
  'show log', 
  'show files', 
  'start sample_file test_argument', 
  'show log', 
  'download sample_file', 
  'savestate testing_state'
  ]

seash.command_loop(command_list)

sys.stdout.close()



# Resets stdout to allow printing to console to allow UTF to catch errors printed by seash
sys.stdout = orig_stdout

command_list = [
  'loadkeys guest0', 
  'as guest0', 
  'loadstate testing_state', 
  'delete example.1.1.r2py', 
  'upload example.1.1.r2py', 
  'reset', 
  'download example.1.1.r2py'
]

seash.command_loop(command_list)



"""
Sets up a basic working state, saves it, and reloads it again to make sure
that seash restores the state of being on a target to execute the command
'show files'.
"""
#pragma out

import seash

command_list = [
  'loadkeys guest0',
  'as guest0',
  'browse',
  'on %1',
  'savestate testing_state'
  ]

seash.command_loop(command_list)


# If loadstate was executed correctly, guest0 should have been restored to being
# on target %1, and the command 'show files' should not throw an error
command_list = [
  'loadkeys guest0',
  'as guest0',
  'loadstate testing_state',
  'show files'
  ]

seash.command_loop(command_list)



"""
Ensure that all modules do not conflict with each other

"""
#pragma out
import os
import seash

import seash_dictionary
import seash_modules
#pragma out Enabled modules: modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)


commands = [
  # Enable all the modules
  'enable geoip',
  'enable factoids',
  'enable variables',
  # Disable all the modules
  'disable geoip',
  'disable factoids',
  'disable variables',
  ]

seash.command_loop(commands)


"""
Performs a few basic commands in creating groups and moving targets around.
No errors or exceptions should be thrown.
"""

import seash
import sys

orig_stdout = sys.stdout

# To temporarily prevent printing to console, test_results.txt is used as a text
# dump as stdout is redirected to it
sys.stdout = open("test_results.txt", "w")

command_list = [
  'loadkeys guest0', 
  'as guest0', 
  'browse', 
  'on %1', 
  'savestate testing_state'
  ] 

seash.command_loop(command_list)

sys.stdout.close()



# No output or errors should be thrown in the following series of commands
sys.stdout = orig_stdout

command_list = [
  'loadkeys guest0', 
  'as guest0', 
  'loadstate testing_state', 
  'add to test_group', 
  'remove %1 from test_group', 
  'add %1 to test_group_2',
  'on %2', 
  'move %2 to test_group'
  ]

seash.command_loop(command_list)



"""
Joins two nodes owned by guest0 and splits them again using resources.offcut
to test if the two commands work properly on the basic scale.
"""
#pragma out
import seash
import sys


command_list = [
  'loadkeys guest3', 
  'as guest3', 
  'browse', 
  'on browsegood', 
  'update', 
  'join', 
  'on %3', 
  'split resources.offcut'
  ]

seash.command_loop(command_list)



"""
Runs a series of set and show commands to make sure no errors are being thrown
during command executions. Due to variations in output caused by the user's
IP address, output results are not being tested too extensively.
"""
import seash
import sys


orig_stdout = sys.stdout

# To temporarily prevent printing to console, test_results.txt is used as a text
# dump as stdout is redirected to it
sys.stdout = open("test_results.txt", "w")

command_list = [
  'loadkeys guest0', 
  'loadkeys guest1', 
  'loadkeys guest2', 
  'as guest1', 
  'browse', 
  'on browsegood', 
  'show info', 
  'update', 
  'show info', 
  'show users', 
  'set users guest0 guest1 guest2', 
  'show users', 
  'show ownerinfo', 
  'set ownerinfo example text testing', 
  'show ownerinfo', 
  'set advertise off', 
  'show advertise', 
  'set advertise on', 
  'show ip', 
  'show ip to test_ip_file.txt', 
  'show hostname',
  'show targets', 
  'show resources', 
  'show offcut', 
  'savestate testing_state', 
  'on %1', 
  'show owner', 
  'set owner guest2', 
  'show owner'
  ]

seash.command_loop(command_list)

sys.stdout.close()



# Overwrites test_results.txt for a clean file for actual testing purposes.
sys.stdout = open("test_results.txt", "w")

command_list = [
  'loadkeys guest1', 
  'as guest1', 
  'loadstate testing_state', 
  'show timeout', 
  'set timeout 123', 
  'show timeout', 
  'show uploadrate', 
  'set uploadrate 2048', 
  'show uploadrate'
  ]

seash.command_loop(command_list)

sys.stdout.close()



# Simple test to see that seash records the values for timeout and uploadrate
# when changed
sys.stdout = orig_stdout

test_results = open("test_results.txt", "r")
wanted_results = open("setandshow_test_results.txt", "r")

original = wanted_results.readlines()
actual = test_results.readlines()

for i in range(len(original)):
  if not original[i] == actual[i]:
    print "Line " + str(i) + " of test_results.txt are not consistent with expected results in setandshow_test_results.txt"

test_results.close()
wanted_results.close()



"""
Author: Alan Loh

Module: A data structure of all the available commands of seash held in a
        dictionary of dictionaries format. Also holds the methods for
        parsing user command input and executing functions corresponding
        to the command.

User input is parsed according to whether or not it follows the structure 
of a command's dictionary and its children.  When parsing a command word, 
it simply checks to see if the user's input string list contains the command word 
at the appropriate index respective to the level of command dictionary currently
being iterated. If it is an user argument, however, it will
simply assign the user's inputted string as the key of the respective
argument's dictionary.

Command input is split by white spaces, so except for the case of arguments 
located at the end of the command string series, the parser will not taken 
into account file names or arguments that are multiple words long. Also, 
because of the way the command callback functions pull out user arguments, 
arguments of similar type need different 'name' field in its command dictionary
to help distinguish one from the other.
"""

# repyportability destroys the built-in type variable, so we need to
# restore the python built-in type function in order for python
# libraries to work.

# The unit tests often import seash_dictionary directly instead of going
# through seash, so let's re-insert here so that we cover as much ground
# as possible..
originaltype = type
import repyportability
import abc
import warnings
abc.type = originaltype
warnings.type = originaltype


# for access to list of targets
import seash_global_variables

import seash_exceptions

import command_callbacks

# Used for module preprocessing of user input
import seash_modules


# Prints the help text of the last command node that has one
# Also lists all sub-commands that follow the command node.
def print_help(input_dict, environment_dict):
  # Iterate through the user command dictionary
  dict_mark = input_dict

  command_string = ""
  cmdgroup = ''

  while dict_mark.keys():
     # Pulls out the current command key word
     command_key = dict_mark.keys()[0]
     if dict_mark[command_key]['name'] == 'display_group':
       cmdgroup = command_key
     else:
       command_string += command_key + ' '

       # If the command's help text isn't an empty string, sets it as the help_text
       # that will be printed out
       if dict_mark[command_key]['help_text'] is not '':
          current_help_text = dict_mark[command_key]['help_text']

     # Iterates into the next dictionary of children commands
     dict_mark = dict_mark[command_key]['children']

  # Remove the 'help' token.
  command_string = command_string.split('help ', 1)[1]

  command_summaries = get_command_summaries(input_dict)
  
  # summary_strings[module] = formatted string for command summary text for all
  #                           commands under that module
  summary_strings = {}
  if command_summaries:
    # cmdgroup can either be a command or a display group.
    # However, if we get to this point, then cmdgroup can never be a command
    # because the command dispatcher will correctly identify it as such.
    # Therefore, if this is not a display group, we should flag it as an error.
    if cmdgroup and not cmdgroup in command_summaries:
      raise seash_exceptions.UserError("'"+cmdgroup+"' is not an enabled command nor a display group.")
    for module in command_summaries[cmdgroup]:
      summary_strings[module] = get_string_from_command_summarylist(command_summaries[cmdgroup][module], command_string)

  # What should be displayed:
  # -- Help text --
  # (help_text that is defined at the command)
  # A target can be either a host:port:vesselname, %ID, or a group name.
  # -- Subcommand list --
  # (Lists the summaries of all subcommands that fall under the display group)
  # add       -- Adds a target (a vessel name or group) to a group.
  # get       -- Acquires vessels from the clearinghouse.
  # release   -- Release vessels from your control
  # show      -- Displays shell state (see 'help show')
  # -- Additional subcommand modifiers --
  # (Lists all additional display groups, if the user did not specify a display group)
  # For more commands, try:
  #   help extended

  print
  print current_help_text.strip()
  print
  
  summary_modules = summary_strings.keys()
  summary_modules.sort()
  for module in summary_modules:
    if not module is None:
      print "Commands from the", module, "module:"
    print summary_strings[module]

  # Display additional cmdgroups if the user did not enter one
  if not cmdgroup:
    # Are there additional keywords?
    additional_cmdgroups = command_summaries.keys()
    # Ignore the default '' keyword.
    if '' in additional_cmdgroups:
      additional_cmdgroups.remove('')
    if None in additional_cmdgroups:
      additional_cmdgroups.remove(None)

    if additional_cmdgroups:
      # List them alphabetically
      additional_cmdgroups.sort()
      print "For more commands, try:"
      command_string = 'help ' + command_string
      for alt_cmdgroup in additional_cmdgroups:
        print "  " + command_string.rstrip(), alt_cmdgroup
      print

"""
Command dictionary entry format:
  '(command key)':{'name':'', 'callback':, 'priority':, 'summary':'', 'example':'', 'help_text':'', 'children':[

'(command key)' - The expected command word the user is suppose to input to
call the command's function. If a certain type of argument is expected, a general
word in all caps should be enclosed within square brackets that signify the type 
of argument needed. For example, '[TARGET]' if a target ID is expected, or 
'[FILENAME]' if the name of a file is needed. Frequently used type includes 
'[TARGET]' for targets, '[KEYNAME]' for loaded keynames, '[FILENAME]' for files,
and '[ARGUMENT]' for everything else, so unless another category of arguments is 
needed, only use those four strings for command keys of arguments in order for 
the parser to work correctly.
 

For general commands like 'browse', however, the key would simply be the same
command word, 'browse'.

In general, the command key should only be a single word from the whole command
string being implemented, and with the exception of arguments that occur at the
end of the command string, no user-inputted arguments should ever contain spaces.


'name':       - The name of the command word. For general commands, it should be 
the same as the command key. For arguments, however, the name should be 
distinguishable from other potential arguments of the same command key to avoid 
conflicts when pulling the user's argument from the input dictionary during 
command execution.


'callback':     - Reference to the command callback function associated with the
command string up to this point. Only command dictionaries that mark a complete 
command string should contain a reference to a callback method. Otherwise, it 
should be set to none. Default location of command callback functions is 
command_callbacks.py.


'priority'      - Gives the command callback function of the dictionary 
containing the key 'priority' the priority of being executed first before 
executing the main function of the command string. It should be implemented and 
assigned True if needed. Otherwise, it should not be added into any other command
dictionary.

An example of how it should work is in the case of 'as [KEYNAME] browse':
A keyname needs to be set before executing 'browse', so the command dictionary of
'[KEYNAME]' has 'priority' in being executed first to set the user's keyname 
before executing 'browse's command function.


'summary'       - A short summary text that should be shown next to the command.
This should be no longer than a sentence if possible. All commands should have a
summary text.


'cmdgroup'- The keyword that should be passed to 'help' to show the summary
text for this command. Commonly used commands that should be shown by default
should have the empty string, or omit this key.


For example, some of the lesser used root commands (join, loadstate, etc.) should
only be shown when the user types 'help extended'. For these commands, the
'cmdgroup' should be set to 'extended'.


'help_text'     - The text that will be outputted whenever a user accesses the
help function for that command. Not every command dictionary needs a help text
associated with it, so it defaults as a blank string, and if none of the command 
dictionaries in the help call holds a help text, it will default at the last 
command dictionary that holds one, namely the dictionary associated with 'help'.


'children'      - The list of command dictionaries that follows the current one.
This will determine the validity of an command input when parsing. Each user
inputted string is verified that it follows one of the potential chains of 
command strings through the series of command dictionaries. Limit only one
argument dictionary per children list to avoid confusion when parsing user
argument input.
 
For example, in the command 'show resources', the children of the command 
dictionary for 'show' will contain the command dictionary 'resources' along with
any other potential command words that can follow 'show'.

"""


seashcommanddict = {
  'on':{
    'name':'on', 'callback':None, 'example': 'target [command]',
    'summary':'Run a command on a target (or changes the default)', 'help_text':"""
on group
on group [command]

Sets the default group for future commands.   Most other commands will only 
operate on the vessels specified in the default group.  The default group is 
listed in the seash command prompt 'identity@group !>'.   The 'on' command can 
also be prepended to another command to set the group for only this command.


Example:

exampleuser@browsegood !> on WAN
exampleuser@WAN !>

exampleuser@browsegood !> on WAN show ip
1.2.3.4
5.6.7.8
exampleuser@browsegood !>
""", 'children':{
      '[TARGET]':{'name':'ontarget', 'callback':command_callbacks.on_target, 'priority':True, 'help_text':'', 'children':{
      }}
  }},


  'as':{
    'name':'as', 'callback':None, 'example':'keyname [command]',
    'summary':'Run a command using an identity (or changes the default)', 'help_text':"""
Sets the default identity for an operation.   The credentials (i.e. public
and private key) for this user are used for the following commands.   
The default identity is listed in the seash command prompt 'identity@group !>'.
The 'as' command can also be prepended to another command to set the identity 
for just this command.

Example:

exampleuser@%all !> as tom
tom@%all !>

exampleuser@browsegood !> as tom browse
(browse output here)
exampleuser@browsegood !>
""", 'children':{
      '[KEYNAME]':{
        'name':'askeyname', 'callback':command_callbacks.as_keyname, 'priority':True,
      'example': '(command)', 'summary': 'Sets the default identity globally or for an operation.',
      'help_text':'', 'children':{
      }},
  }},


  'help':{
    'name':'help', 'callback':print_help, 'priority':True,
    'cmdgroup':None, 'help_text':"""
A target can be either a host:port:vesselname, %ID, or a group name.

See https://seattle.cs.washington.edu/wiki/RepyTutorial for more info!""",
    'children':{}},
  'show':{
    'name':'show', 'callback':command_callbacks.show,
    'summary': "Displays the shell state (see 'help show')", 'help_text':"""
Displays information regarding the current state of Seattle, depending on
the additional keywords that are passed in.

  (*) No need to update prior, the command contacts the nodes anew

""", 'children':{
      'info':{
        'name':'info', 'callback':command_callbacks.show_info,
        'summary':'Display general information about the vessels','help_text':"""
show info

This command prints general information about vessels in the default group
including the version, nodeID, etc.

Example:
exampleuser@%1 !> show info
192.x.x.178:1224:v3 has no information (try 'update' or 'list')
exampleuser@%1 !> update
exampleuser@%1 !> show info
192.x.x.178:1224:v3 {'nodekey': {'e': 65537L, 'n': 929411623458072017781884599109L}, 'version': '0.1r', 'nodename': '192.x.x.175'}

""", 'children':{}},
      'users':{
        'pattern':'users', 'name':'users', 'callback':command_callbacks.show_users,
        'summary':'Display the user keys for the vessels','help_text':"""
show users

This command lists the set of user keys for vessels in the default group.   
If the key has been loaded into seash as an identity, this name will be used.

Example:
exampleuser@browsegood !> show users
192.x.x.178:1224:v3 has no information (try 'update' or 'list')
192.x.x.2:1224:v12 has no information (try 'update' or 'list')
192.x.x.2:1224:v3 has no information (try 'update' or 'list')
exampleuser@browsegood !> update
exampleuser@browsegood !> show users
192.x.x.178:1224:v3 (no keys)
192.x.x.2:1224:v12 65537 136475...
192.x.x.2:1224:v3 exampleuser

""", 'children':{}},
      'ownerinfo':{
        'name':'ownerinfo', 'callback':command_callbacks.show_ownerinfo,
        'summary':'Display owner information for the vessels','help_text':"""
show ownerinfo

This lists the ownerinfo strings for vessels in the default group.   See
'set ownerinfo' for more details
""", 'children':{}},
      'advertise':{
        'name':'advertise', 'callback':command_callbacks.show_advertise,
        'summary':"Shows whether the node manager will advertise the vessel's keys in the advertise services.", 'help_text':"""
show advertise

This indicates whether the node manager will advertise the vessel's keys in 
the advertise services.   See 'set advertise' for more details.
""", 'children':{}},
      'ip':{
        'name':'ip', 'callback':command_callbacks.show_ip, 'example': '[to file]',
        'summary':'Display the ip addresses of the nodes', 'help_text':"""
show ip 
show ip [to file]

This lists the ip addresses of the vessels in the default group.   These IP
addresses may be optionally written to a file.   

Note that machines behind a NAT, mobile devices, or other systems with 
atypical network connectivity may list a host name instead.

Example:
exampleuser@ !> show targets
browsegood ['192.x.x.2:1224:v12', '192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%4 ['219.x.x.62:1224:v4']
%all ['192.x.x.2:1224:v12', '192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%1 ['192.x.x.2:1224:v12']
%3 ['193.x.x.42:1224:v18']
%2 ['192.x.x.2:1224:v3']
exampleuser@ !> on browsegood
exampleuser@browsegood !> show ip
192.x.x.2
193.x.x.42
219.x.x.62

""", 'children':{
          'to':{
            'name':'to', 'callback':None, 'example': '[filename]',
            'summary': 'Outputs the list of ip addresses of the vessels in the default group to [filename]',
            'help_text':'', 'children':{
              '[FILENAME]':{'name':'filename', 'callback':command_callbacks.show_ip_to_file, 'help_text':'', 'children':{}},
          }},
          '>':{
            'name':'>', 'callback':None, 'example': '[filename]',
            'summary': 'Outputs the list of ip addresses of the vessels in the default group to [filename]',
            'help_text':'', 'children':{
              '[FILENAME]':{'name':'filename', 'callback':command_callbacks.show_ip_to_file, 'help_text':'', 'children':{}},
          }},
      }},
      'hostname':{
        'name':'hostname', 'callback':command_callbacks.show_hostname,
        'summary':'Display the hostnames of the nodes','help_text':"""
show hostname

This lists the DNS host names for the vessels in the default group.   If this
information is not available, this will be listed.

Example:
exampleuser@browsegood !> show ip
192.x.x.2
193.x.x.42
219.x.x.62
exampleuser@browsegood !> show hostname
192.x.x.2 is known as Guest-Laptop.home
193.x.x.42 is known as pl2.maskep.aerop.fr
219.x.x.62 has unknown host information

""", 'children':{}},
      'owner':{'name':'owner', 'callback':command_callbacks.show_owner, 'help_text':"""
show owner

Displays the owner key (or identity if known) for the vessels in the default
group.

Example:
exampleuser@ !> show targets
browsegood ['192.x.x.2:1224:v12', '192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%4 ['219.x.x.62:1224:v4']
%all ['192.x.x.2:1224:v12', '192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%1 ['192.x.x.2:1224:v12']
%3 ['193.x.x.42:1224:v18']
%2 ['192.x.x.2:1224:v3']
exampleuser@ !> on browsegood
exampleuser@browsegood !> show owner
192.x.x2:1224:v12 exampleuser pubkey
192.x.x.2:1224:v3 65537 127603...
193.x.x.42:1224:v18 65537 163967...
219.x.x.62:1224:v4 65537 952875...

""", 'children':{}},
      'targets':{
        'name':'targets', 'callback':command_callbacks.show_targets,
        'summary':'Display a list of targets','help_text':"""
show targets

Lists the known targets (groups and individual nodes) that commands may be 
run on.

Example:
exampleuser@ !> show targets
%all (empty)
exampleuser@ !> browse
['192.x.x.2:1224', '219.x.x.62:1224', '193.x.x.42:1224']
Added targets: %3(193.x.x.42:1224:v18), %4(219.x.x.62:1224:v4), %1(192.x.x.2:1224:v12), %2(192.x.x.2:1224:v3)
Added group 'browsegood' with 4 targets
yaluen@ !> show targets
browsegood ['192.x.x.2:1224:v12', '192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%4 ['219.x.x.62:1224:v4']
%all ['192.x.x.2:1224:v12', '192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%1 ['192.x.x.2:1224:v12']
%3 ['193.x.x.42:1224:v18']
%2 ['192.x.x.2:1224:v3']

""", 'children':{}},
      'groups':{
        'name':'groups', 'callback':command_callbacks.show_groups,
        'summery':'Display a list of groups','help_text':"""
show groups

Lists available groups.

Example:
exampleuser@ !> show groups
%all []
exampleuser@ !> browse
['192.x.x.2:1224', '219.x.x.62:1224', '193.x.x.42:1224']
Added targets: %3(193.x.x.42:1224:v18), %4(219.x.x.62:1224:v4), %1(192.x.x.2:1224:v12), %2(192.x.x.2:1224:v3)
Added group 'browsegood' with 4 targets
exampleuser@ !> show groups
browsegood ['192.x.x.2:1224:v12', '192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%all ['192.x.x.2:1224:v12', '192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']

""", 'children':{}},
      'identities':{
        'name':'identities', 'callback':command_callbacks.show_identities,
        'summary':'Display the known identities', 'help_text':"""
show identities

Lists the identities loaded into the shell and whether the public or private
keys are loaded.   This does not display the keys themselves (see 'show keys').

Example:
 !> show identities
 !> loadkeys exampleuser
 !> loadkeys guest0
 !> loadkeys guest1
 !> show identities
guest2 PRIV
exampleuser PUB PRIV
guest0 PUB PRIV
guest1 PUB PRIV

""", 'children':{}},
      'keys':{
        'name':'keys', 'callback':command_callbacks.show_keys,
        'summary':'Display the known keys', 'help_text':"""
show keys

List the actual keys loaded by the shell.   To see identity information, see
'show identities'.

Example:
 !> show keys
 !> loadkeys yaluen
 !> loadpub guest0
 !> loadpriv guest1
 !> show keys
exampleuser {'e': 65537L, 'n': 967699203053798948061567293973111925102424779L} {'q': 130841985099129780748709L, 'p': 739593793918579524787167931524344434698161314501292256851220768397231L, 'd': 9466433905223884723074560052831388470409993L}
guest0 {'e': 65537L, 'n': 9148459067481753275566379538357634516166379961L} None
guest1 None {'q': 121028014346935113507847L, 'p': 107361553689073802754887L, 'd': 127298628609806695961451784003754746302524139001L}

""", 'children':{}},
      'log':{
        'name':'log', 'callback':command_callbacks.show_log,
        'example': '[to file]', 'summary':'Display the log from the vessel (*)', 'help_text':"""
Lists the log of operations from the vessel.   This log is populated by print
statements and exceptions from the program running in the vessel.
""", 'children':{
          'to':{
            'name':'to', 'callback':None, 'example': '[filename]',
            'summary': 'Writes the log to a file.', 'help_text':'', 'children':{
              '[FILENAME]':{'name':'filename', 'callback':command_callbacks.show_log_to_file, 'help_text':'', 'children':{}},
          }},
      }},
      'files':{
        'name':'files', 'callback':command_callbacks.show_files,
        'summary':'Display a list of files in the vessel (*)', 'help_text':"""
show files

Lists the names of the files loaded into vessels in the default groups.   
This is similar to dir or ls.

Example:
exampleuser@browsegood !> show files
Files on '192.x.x.2:1224:v3': ''
Files on '193.x.x.42:1224:v18': ''
Files on '219.x.x.62:1224:v4': ''
exampleuser@browsegood !> upload example.1.1.r2py
exampleuser@browsegood !> show files
Files on '192.x.x.2:1224:v3': 'example.1.1.r2py'
Files on '193.x.x.42:1224:v18': 'example.1.1.r2py'
Files on '219.x.x.62:1224:v4': 'example.1.1.r2py'

""", 'children':{}},
      'resources':{
        'name':'resources', 'callback':command_callbacks.show_resources,
        'summary':'Display the resources / restrictions for the vessel (*)', 'help_text':"""
show resources

Lists the resources allotted to vessels in the default group.
""", 'children':{}},
      'offcut':{
        'name':'offcut', 'callback':command_callbacks.show_offcut,
        'summary':'Display the offcut resource for the node (*)', 'help_text':"""
show offcut

This lists the amount of resources that will be lost by splitting a vessel or
gained by joining two vessels.   This is shown on a per-node basis amongst
all vessels in the default group
""", 'children':{}},
      'timeout':{
        'name':'timeout', 'callback':command_callbacks.show_timeout,
        'summary':'Display the timeout for nodes', 'help_text':"""
show timeout

This shows the amount of time the shell will wait for a command to timeout.   
Note that commands like 'run' and 'upload' will use both this value and the
uploadrate setting

Example:
 !> show timeout
10

""", 'children':{}},
      'uploadrate':{
        'name':'uploadrate', 'callback':command_callbacks.show_uploadrate,
        'summary':'Display the upload rate which seash uses to estimate the required time for a file upload', 'help_text':"""
show uploadrate

This lists the minimum rate at which the shell should allow uploads to occur.
Uploads to vessels that go slower than this will be aborted.   Note that this
is used in combination with the timeout setting.

Example:
 !> show uploadrate
102400

""", 'children':{}},
  }},


  'run':{
    'name':'run', 'callback':None, 'example':'file [args ...]',
    'summary':'Upload a file and start executing it', 'help_text':"""
Uploads a program to a vessel and starts it running.   (This command is
actually just a short-cut for the 'upload' and 'start' commands).   The
arguments listed will be passed to the command when it is started.

This command will make an educated guess as to what platform your
program is written for (i.e. repyV1 or repyV2).  You can override this
by using 'runv1' or 'runv2', respectively.

Example:
exampleuser@browsegood !> show log
Log from '192.x.x.2:1224:v3':

Log from '193.x.x.42:1224:v18':

Log from '219.x.x.62:1224:v4':

Log from '192.x.x.2:1224:v12':

exampleuser@browsegood !> run example.1.1.r2py
exampleuser@browsegood !> show log
Log from '192.x.x.2:1224:v3':
Hello World

Log from '193.x.x.42:1224:v18':
Hello World

Log from '219.x.x.62:1224:v4':
Hello World

Log from '192.x.x.2:1224:v12':
Hello World

""", 'children':{
      '[FILENAME]':{
        'name':'filename', 'callback':command_callbacks.run_localfn, 'example':'[arg1, arg2, ...]',
        'summary': 'Uploads the file to the vessels and starts running them, passing arguments if specified.',
        'help_text':'','children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.run_localfn_arg, 'help_text':'', 'children':{}},
      }},
  }},

  'runv1':{
    'name':'run', 'callback':None, 'example':'file [args ...]',
    'summary':'Upload a file and start executing it as repyV1', 'help_text':"""
Uploads a program to a vessel and starts it running.   (This command is
actually just a short-cut for the 'upload' and 'startv1' commands).
The arguments listed will be passed to the command when it is started.

Example:
exampleuser@browsegood !> show log
Log from '192.x.x.2:1224:v3':

Log from '193.x.x.42:1224:v18':

Log from '219.x.x.62:1224:v4':

Log from '192.x.x.2:1224:v12':

exampleuser@browsegood !> run example.1.1.r2py
exampleuser@browsegood !> show log
Log from '192.x.x.2:1224:v3':
Hello World

Log from '193.x.x.42:1224:v18':
Hello World

Log from '219.x.x.62:1224:v4':
Hello World

Log from '192.x.x.2:1224:v12':
Hello World

""", 'children':{
      '[FILENAME]':{
        'name':'filename', 'callback':command_callbacks.run_localfn, 'example':'[arg1, arg2, ...]',
        'summary': 'Uploads the file to the vessels and starts running them, passing arguments if specified.',
        'help_text':'','children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.run_localfn_arg, 'help_text':'', 'children':{}},
      }},
  }},


  'runv2':{
    'name':'run', 'callback':None, 'example':'file [args ...]',
    'summary':'Upload a file and start executing it as repyV2', 'help_text':"""
Uploads a program to a vessel and starts it running.   (This command is
actually just a short-cut for the 'upload' and 'startv2' commands).
The arguments listed will be passed to the command when it is started.

Example:
exampleuser@browsegood !> show log
Log from '192.x.x.2:1224:v3':

Log from '193.x.x.42:1224:v18':

Log from '219.x.x.62:1224:v4':

Log from '192.x.x.2:1224:v12':

exampleuser@browsegood !> run example.1.1.r2py
exampleuser@browsegood !> show log
Log from '192.x.x.2:1224:v3':
Hello World

Log from '193.x.x.42:1224:v18':
Hello World

Log from '219.x.x.62:1224:v4':
Hello World

Log from '192.x.x.2:1224:v12':
Hello World

""", 'children':{
      '[FILENAME]':{
        'name':'filename', 'callback':command_callbacks.run_localfn, 'example':'[arg1, arg2, ...]',
        'summary': 'Uploads the file to the vessels and starts running them, passing arguments if specified.',
        'help_text':'','children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.run_localfn_arg, 'help_text':'', 'children':{}},
      }},
  }},

  'runv1':{
    'name':'run', 'callback':None, 'example':'file [args ...]',
    'summary':'Upload a file and start executing it as repyV1', 'help_text':"""
Uploads a program to a vessel and starts it running.   (This command is
actually just a short-cut for the 'upload' and 'startv1' commands).
The arguments listed will be passed to the command when it is started.

Example:
exampleuser@browsegood !> show log
Log from '192.x.x.2:1224:v3':

Log from '193.x.x.42:1224:v18':

Log from '219.x.x.62:1224:v4':

Log from '192.x.x.2:1224:v12':

exampleuser@browsegood !> run example.1.1.r2py
exampleuser@browsegood !> show log
Log from '192.x.x.2:1224:v3':
Hello World

Log from '193.x.x.42:1224:v18':
Hello World

Log from '219.x.x.62:1224:v4':
Hello World

Log from '192.x.x.2:1224:v12':
Hello World

""", 'children':{
      '[FILENAME]':{
        'name':'filename', 'callback':command_callbacks.run_localfn, 'example':'[arg1, arg2, ...]',
        'summary': 'Uploads the file to the vessels and starts running them, passing arguments if specified.',
        'help_text':'','children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.run_localfn_arg, 'help_text':'', 'children':{}},
      }},
  }},


  'runv2':{
    'name':'run', 'callback':None, 'example':'file [args ...]',
    'summary':'Upload a file and start executing it as repyV2', 'help_text':"""
Uploads a program to a vessel and starts it running.   (This command is
actually just a short-cut for the 'upload' and 'startv2' commands).
The arguments listed will be passed to the command when it is started.

Example:
exampleuser@browsegood !> show log
Log from '192.x.x.2:1224:v3':

Log from '193.x.x.42:1224:v18':

Log from '219.x.x.62:1224:v4':

Log from '192.x.x.2:1224:v12':

exampleuser@browsegood !> run example.1.1.r2py
exampleuser@browsegood !> show log
Log from '192.x.x.2:1224:v3':
Hello World

Log from '193.x.x.42:1224:v18':
Hello World

Log from '219.x.x.62:1224:v4':
Hello World

Log from '192.x.x.2:1224:v12':
Hello World

""", 'children':{
      '[FILENAME]':{
        'name':'filename', 'callback':command_callbacks.run_localfn, 'example':'[arg1, arg2, ...]',
        'summary': 'Uploads the file to the vessels and starts running them, passing arguments if specified.',
        'help_text':'','children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.run_localfn_arg, 'help_text':'', 'children':{}},
      }},
  }},


  'add':{
    'name':'add', 'callback':None, 'example': '[target] [to group]',
    'summary':'Adds a target (a vessel name or group) to a group', 'help_text':"""
add target [to group]
add to group

Adds a target (a vessel name or group) a group.   If the group does not exist,
it is created.   This can be used to control which vessels are manipulated by 
different commands.   The short form 'add target' adds the target to the 
default group.   The short form 'add to group' adds the default group to
the target.

If the target is already in the group, an error message will be printed.

Example:
exampleuser@%1 !> on new_group
Invalid command input: Target does not exist
exampleuser@%1 !> add to new_group
exampleuser@%1 !> add %2 to new_group
exampleuser@%1 !> on new_group
exampleuser@new_group !> list
  ID Own                      Name     Status              Owner Information
  %1  *        192.x.x.178:1224:v3      Fresh                               
  %2  *         192.x.x.2:1224:v12      Fresh    

""", 'children':{
      '[TARGET]':{
        'name':'target', 'callback':command_callbacks.add_target,
        'example': 'to group', 'summary': 'Adds a target to a group.', 'help_text':'', 'children':{
          'to':{'name':'to', 'callback':None, 'help_text':'', 'children':{
              '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.add_target_to_group, 'help_text':'', 'children':{}},
          }},
      }},
      'to':{
        'name':'to', 'callback':None, 'example': 'group',
        'summary':'Adds the default target to a group.', 'help_text':'', 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.add_to_group, 'help_text':'', 'children':{}},
      }},
  }},


  'move':{
    'name':'move', 'callback':None, 'example': 'target to group',
    'summary': 'Add target to group, remove target from default', 'cmdgroup': 'extended', 'help_text':"""
This is essentially a shortcut for removing the target from the default group
and adding it to group.   See 'add' and 'remove' for more information.
""", 'children':{
      '[TARGET]':{
        'name':'target', 'callback':None, 'example': 'to group',
        'summary': 'Moves the target to the group', 'help_text':'', 'children':{
          'to':{'name':'to', 'callback':None, 'help_text':'', 'children':{
              '[GROUP]':{'name':'group', 'callback':command_callbacks.move_target_to_group, 'help_text':'', 'children':{}},
          }},
      }},
  }},


  'remove':{
    'name':'remove', 'callback':None, 'example':'[target] [from group]',
    'summary':'Removes a target from a group', 'help_text':"""
remove [target] from group

This command removes a target (vesselname or group) from a group.   This means
that future group operations will not include the listed vesselname or group.
The short form 'remove target' removes the target from the default group.
The short form 'remove from group' removes the default group from group.

If the target is not in the group, an error message will be printed.

Example:
exampleuser@new_group !> list
  ID Own                      Name     Status              Owner Information
  %1  *        192.x.x.178:1224:v3      Fresh                               
  %2  *         192.x.x.2:1224:v12      Fresh    
  %3             192.x.x.2:1224:v3      Fresh      
exampleuser@new_group !> on %1
exampleuser@%1 !> remove from new_group
exampleuser@%1 !> remove %2 from new_group
exampleuser@%1 !> on new_group
exampleuser@new_group !> list
  ID Own                      Name     Status              Owner Information
  %3             192.x.x.2:1224:v3      Fresh   

""", 'children':{
      '[TARGET]':{
        'name':'target', 'callback':command_callbacks.remove_target, 'example':'from group',
        'summary': 'Removes the target from the default group', 'help_text':'', 'children':{
          'from':{'name':'from', 'callback':None, 'help_text':'', 'children':{
            '[GROUP]':{
              'name':'group', 'callback':command_callbacks.remove_target_from_group,
              'summary': 'Removes the target from the group', 'help_text':'',
              'children':{}},
          }},
      }},
      'from':{
        'name':'from', 'callback':None, 'example': 'group',
        'summary': 'Removes the default group from group', 'help_text':'', 'children':{
          '[GROUP]':{'name':'group', 'callback':command_callbacks.remove_from_group, 'help_text':'', 'children':{}},
      }},
  }},


  'set':{
    'name':'set', 'callback':command_callbacks.set,
    'summary': "Changes the shell or vessels (see 'help set')", 'help_text':"""
Changes the shell or vessels.
""", 'children':{
      'users':{
        'name':'users', 'callback':None, 'example':'[identity ...]',
        'summary':"Change a vessel's users", 'help_text':"""
set users [identity1, identity2, ...]

Sets the user keys for vessels in the default group.   The current identity
must own the vessels.

Example:
exampleuser@%1 !> show owner
192.x.x.2:1224:v12 exampleuser pubkey
exampleuser@%1 !> show users
192.x.x.2:1224:v12 65537 136475...
exampleuser@%1 !> set users guest0 guest1 guest2
exampleuser@%1 !> update
exampleuser@%1 !> show users
192.x.x.2:1224:v12 guest0 guest1 guest2
exampleuser@%1 !> on %2
exampleuser@%2 !> show owner
192.x.x.2:1224:v3 65537 127603...
exampleuser@%2 !> set users guest0 guest1
Failure 'Node Manager error 'Insufficient Permissions'' on  192.x.x.2:1224:v3

""", 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.set_users_arg, 'help_text':'', 'children':{}},
      }},
      'ownerinfo':{
        'name':'ownerinfo', 'callback':None, 'example': '[ data ... ]',
        'summary': 'Change owner information for the vessels', 'help_text':"""
set ownerinfo 'string'

This command sets the owner information for each vessel in the default group.
The default identity must own the vessels.

Example:
exampleuser@browsegood !> show owner
192.x.x.2:1224:v12 exampleuser pubkey
192.x.x.2:1224:v3 65537 127603...
exampleuser@browsegood !> show ownerinfo
192.x.x.2:1224:v12 ''
192.x.x.2:1224:v3 ''
exampleuser@browsegood !> set ownerinfo Example owner info
Failure 'Node Manager error 'Insufficient Permissions'' on  192.x.x.2:1224:v3
Added group 'ownerinfogood' with 1 targets and 'ownerinfofail' with 1 targets
exampleuser@browsegood !> update
exampleuser@browsegood !> show ownerinfo
192.x.x.2:1224:v12 'Example owner info'
192.x.x.2:1224:v3 ''

""", 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.set_ownerinfo_arg, 'help_text':'', 'children':{}},
      }},
      'advertise':{
        'name':'advertise', 'callback':None,
        'summary':'Change advertisement information about the vessels','help_text':"""
set advertise [on/off]

This setting is changable only by the vessel owner and indicates whether or
not the node's IP / port should be advertised under the owner and user keys.
The default value is on.   With this turned off, the 'browse' command will
be unable to discover the vessel.

exampleuser@%1 !> show owner
192.x.x.2:1224:v12 exampleuser pubkey
exampleuser@%1 !> show advertise
192.x.x.2:1224:v12 on
exampleuser@%1 !> set advertise off
exampleuser@%1 !> update
exampleuser@%1 !> show advertise
192.x.x.2:1224:v12 off
exampleuser@%1 !> on %2
exampleuser@%2 !> show owner
192.x.x.2:1224:v3 65537 127603...
exampleuser@%2 !> show advertise
192.x.x.2:1224:v3 on
exampleuser@%2 !> set advertise off
Failure 'Node Manager error 'Insufficient Permissions'' on  192.x.x.2:1224:v3

""", 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.set_advertise_arg, 'help_text':'', 'children':{}},
      }},
      'owner':{
        'name':'owner', 'callback':None, 'example': 'identity',
        'summary': "Changes a vessel's owner.", 'help_text':"""
set owner identity

This changes the owner key for all vessels in the default group to the
identity specified.   This command may only be issued by the vessels' current
owner.

Example:
exampleuser@%1 !> show identities
exampleuser PUB PRIV
guest0 PUB PRIV
guest1 PUB PRIV
exampleuser@%1 !> show owner
192.x.x.2:1224:v12 exampleuser pubkey
exampleuser@%1 !> set owner guest0
exampleuser@%1 !> update
exampleuser@%1 !> show owner
192.x.x.2:1224:v12 guest0 pubkey
exampleuser@%1 !> on %2
exampleuser@%2 !> show owner
192.x.x.2:1224:v3 65537 127603...
exampleuser@%2 !> set owner exampleuser
Failure 'Node Manager error 'Insufficient Permissions'' on  192.x.x.2:1224:v3

""", 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.set_owner_arg, 'help_text':'', 'children':{}},
      }},
      'timeout':{
        'name':'timeout', 'callback':None, 'example': 'count',
        'summary': 'Sets the time that seash is willing to wait on a node',
        'help_text':"""
set timeout timeoutval

This sets the timeout for network related commands.   Most commands will then
be aborted on nodes if they are not completed withing the timeoutval number of
seconds.   Note that the upload and run commands also use the uploadrate 
setting to determine their timeout.

Example:
exampleuser@%1 !> set timeout 1
exampleuser@%1 !> show timeout
1
exampleuser@%1 !> start example.1.1.r2py
Failure 'signedcommunicate failed on session_recvmessage with error 'recv() timed out!'' uploading to 193.x.x.42:1224:v18
exampleuser@%1 !> set timeout 10
exampleuser@%1 !> start example.1.1.r2py
exampleuser@%1 !> show log
Log from '193.x.x.42:1224:v18':
Hello World

""", 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.set_timeout_arg, 'help_text':'', 'children':{}},
      }},
      'uploadrate':{
        'name':'uploadrate', 'callback':None, 'example': 'speed',
        'summary': 'Sets the upload rate which seash will use to estimate the time needed for a file to be uploaded to a vessel. The estimated time would be set as the temporary timeout count during actual process. Speed should be in bytes/sec.',
        'help_text':"""
set uploadrate rate_in_bps

This value is used along with the timeout value to determine when to declare an
upload or run command as failed.   The wait time is computed as:
   timeout + filesize / rate_in_bps

Thus if the timeout is 10 seconds and rate_in_bps is 102400 (100 KB / s), a 1MB 
will attempt to upload for 20 seconds.

Example:
exampleuser@%1 !> set uploadrate 99999999999999
exampleuser@%1 !> set timeout 1
exampleuser@%1 !> upload example.1.1.r2py
Failure 'signedcommunicate failed on session_recvmessage with error 'recv() timed out!'' uploading to 193.x.x.42:1224:v18
exampleuser@%1 !> set uploadrate 102400
exampleuser@%1 !> upload example.1.1.r2py
exampleuser@%1 !> 

""", 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.set_uploadrate_arg, 'help_text':'', 'children':{}},
      }},
      'autosave':{
        'name':'autosave', 'callback':None, 'example': '[ on | off ]',
        'summary': "Sets whether to save the state after every command. Set to 'off' by default. The state is saved to a file called 'autosave_keyname', where keyname is the name of the current key you're using.",
        'help_text':"""
set autosave [on/off]

When turned on, the shell settings such as keys, targets, timeout value, etc.
will all be persisted to disk after every operation.   These are saved in a
file called 'autosave_(user's keyname)', which is encrypted with the default identity.   The user
can then restore the shell's state by typing 'loadstate identity'.

Example:
exampleuser@%1 !> set autosave on
exampleuser@%1 !> exit
(restart seash.py)
 !> loadkeys exampleuser
 !> as exampleuser
exampleuser@ !> loadstate autosave_exampleuser
exampleuser@%1 !>

""", 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.set_autosave_arg, 'help_text':'', 'children':{}},
      }},
      'showparse': {
        'name':'showparse', 'callback': None, 'example': '[ on | off ]', 
        'summary': 'Toggle display of parsed command line input', 'help_text':"""
set showparse [on/off]

When turned on, when any modules modify the command line input, the resulting
input will be printed to the screen.

Example:
exampleuser@%1 !> set showparse on
exampleuser@%1 !> set username exampleuser2
exampleuser@%1 !> as $username
as exampleuser2
exampleuser2@%1 !> set showparse off
exampleuser2@%1 !> as $username
exampleuser2@%1 !>

""", 'children': {
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.set_showparse_args, 'help_text':'', 'children':{}},
      }},
  }},


  'browse':{
    'name':'browse', 'callback':command_callbacks.browse,
    'summary': 'Find vessels I can control', 'help_text':"""
browse [advertisetype]

This command will use the default identity to search for vessels that can
be controlled.   Any vessel with the advertise flag set will be advertised
in at least one advertise service.   browse will look into these services
and add any vessels it can contact.

Setting advertisetype will restrict the advertise lookup to only use that 
service.   Some permitted values for advertisetype are central, DHT, and DOR.

Example:
exampleuser@ !> show targets
%all (empty)
exampleuser@ !> browse
['192.x.x.2:1224', '193.x.x.42:1224', '219.x.x.62:1224']
Added targets: %2(193.x.x.42:1224:v18), %3(219.x.x.62:1224:v4), %1(192.x.x.2:1224:v3)
Added group 'browsegood' with 3 targets
exampleuser@ !> show targets
browsegood ['192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%3 ['219.x.x.62:1224:v4']
%all ['192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%1 ['192.x.x.2:1224:v3']
%2 ['193.x.x.42:1224:v18']

""", 'children':{
      '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.browse_arg, 'help_text':'', 'children':{}},
  }},


  'genkeys':{
    'name':'genkeys', 'callback':None, 'example':'fn [len] [as identity]',
    'summary':'Creates new pub / priv keys (default len=1024)', 'cmdgroup': 'extended', 'help_text':"""
genkeys keyprefix [as identity]

Generates a new set of keys, writing them to files keyprefix.publickey and
keyprefix.privatekey.   It also adds the identity under the name given.  If
identity is not specified, keyprefix is used.

Example:
 !> genkeys userA as userB
Created identity 'userB'
 !> show identities
userB PUB PRIV
 !> loadkeys userA
 !> show identities
userB PUB PRIV
userA PUB PRIV

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.genkeys_filename, 'help_text':'', 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.genkeys_filename_len, 'help_text':'', 'children':{
              'as':{'name':'as', 'callback':None, 'help_text':'', 'children':{
                  '[ARGUMENT]':{'name':'keyname', 'callback':command_callbacks.genkeys_filename_len_as_identity, 'help_text':'', 'children':{}},
              }},
          }},
          'as':{'name':'as', 'callback':None, 'help_text':'', 'children':{
              '[ARGUMENT]':{'name':'keyname', 'callback':command_callbacks.genkeys_filename_as_identity, 'help_text':'', 'children':{}},
          }},
      }},
  }},


  'loadkeys':{
    'name':'loadkeys', 'callback':None, 'example': 'fn [as identity]',
    'summary':'Loads filename.publickey and filename.privatekey', 'help_text':"""
loadkeys keyprefix [as identity]

Loads a public key named keyprefix.publickey and a private key named
keyprefix.privatekey.   This is a shortcut for the 'loadpub' and 'loadpriv'
operations.   If identity is specified, the shell refers to the keys using
this.   If not, keyprefix is the identity.

Example:
 !> loadkeys exampleuser
 !> show identities
exampleuser PUB PRIV
 !> as exampleuser
exampleuser@ !>

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.loadkeys_keyname, 'help_text':'', 'children':{
          'as':{'name':'as', 'callback':None, 'help_text':'', 'children':{
              '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.loadkeys_keyname_as, 'help_text':'', 'children':{}},
          }},
      }},
  }},


  'loadpub':{
    'name':'loadpub', 'callback':None, 'example':'fn [as identity]',
    'summary':'Loads filename.publickey', 'cmdgroup': 'extended', 'help_text':"""
loadpub pubkeyfile [as identity]

Loads a public key named keyprefix.publickey.  If identity is specified, the 
shell refers to the keys using this.   If not, keyprefix is the identity.

Example:
 !> loadpub exampleuser
 !> show identities
exampleuser PUB

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.loadpub_filename, 'help_text':'', 'children':{
          'as':{'name':'as', 'callback':None, 'help_text':'', 'children':{
              '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.loadpub_filename_as, 'help_text':'', 'children':{}},
          }},
      }},
  }},


  'loadpriv':{
    'name':'loadpriv', 'callback':None, 'example':'fn [as identity]',
    'summary':'Loads filename.privatekey', 'cmdgroup': 'extended',
    'help_text':"""
loadpriv privkeyfile [as identity]

Loads a private key named keyprefix.privatekey.  If identity is specified, the 
shell refers to the keys using this.   If not, keyprefix is the identity.

Example:
 !> loadpriv exampleuser
 !> show identities
exampleuser PRIV

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.loadpriv_filename, 'help_text':'', 'children':{
          'as':{'name':'as', 'callback':None, 'help_text':'', 'children':{
              '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.loadpriv_filename_as, 'help_text':'', 'children':{}},
          }},
      }},
  }},

  
  'list':{
    'name':'list', 'callback':command_callbacks.list,
    'summary':'Update and display information about the vessels', 'help_text':"""
list

Display status information about a set of vessels.   This indicates whether
the vessels are running programs, if the default identity is the owner or
just a user, along with other useful information.

Example:
exampleuser@browsegood !> list
  ID Own                      Name     Status              Owner Information
  %1  *        192.x.x.178:1224:v3      Fresh                               
  %2  *         192.x.x.2:1224:v12      Fresh                               
  %3             192.x.x.2:1224:v3      Fresh  

""", 'children':{}},


  'upload':{
    'name':'upload', 'callback':None, 'example':'localfn (remotefn)',
    'summary': 'Upload a file', 'help_text':"""
upload srcfilename [destfilename]

Uploads a file into all vessels in the default group.   The file name that is
created in those vessels is destfilename (or srcfilename by default).

Example:
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': ''
exampleuser@%1 !> upload example.1.1.r2py
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': 'example.1.1.r2py'

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.upload_filename, 'help_text':'', 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.upload_filename_remotefn, 'help_text':'', 'children':{}}
      }},
  }},


  'download':{
    'name':'download', 'callback':None, 'example':'remotefn (localfn)',
    'summary': 'Download a file (to multiple local files)', 'help_text':"""
download srcfilename [destfilename]

Retrieves a copy of srcfilename from every vessel in the default group.   The
file is written as the destfilename.vesselname (with ':' replaced with '_').
If the destfilename is not specified, srcfilename is used instead.

Example:
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': 'example.1.1.r2py'
exampleuser@%1 !> download example.1.1.r2py
Wrote files: example.1.1.r2py.192.x.x.2_1224_v3 
yaluen@%1 !> download example.1.1.r2py test_download
Wrote files: test_download.192.x.x.2_1224_v3 

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.download_filename, 'help_text':'', 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.download_filename_localfn, 'help_text':'', 'children':{}}
      }},
  }},


  'delete':{
    'name':'delete', 'callback':None, 'example': 'remotefn',
    'summary': 'Delete a file.', 'help_text':"""
delete filename

Erases filename from every vessel in the default group.

Example:
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': 'example.1.1.r2py'
exampleuser@%1 !> delete example.1.1.r2py
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': ''

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.delete_remotefn, 'help_text':'', 'children':{}},
  }},

  'cat':{
    'name':'cat', 'callback':None, 'example':'remotefn',
    'summary':'Display the contents of a remote file', 'help_text':"""
cat filename

Displays the content of filename from every vessel in the default group.

Example:
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': 'example.1.1.r2py'
exampleuser@%1 !> cat example.1.1.r2py

File 'example.1.1.r2py' on '192.168.1.2:1224:v3': 
if callfunc == 'initialize':
  print "Hello World"

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.cat_filename, 'help_text':'', 'children':{}},
  }},


  'reset':{
    'name':'reset', 'callback':command_callbacks.reset,
    'summary':'Reset the vessel (clear files / log and stop)', 'help_text':"""
reset

Clears the log, stops any programs, and deletes all files from every vessel
in the default group.

Example:
exampleuser@%1 !> show log
Log from '192.x.x.2:1224:v3':
Hello World

exampleuser@%1 !> list
  ID Own                      Name     Status              Owner Information
  %1           192.x.x.2:1224:v3 Terminated                               
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': 'example.1.1.r2py'
exampleuser@%1 !> reset
exampleuser@%1 !> show log
Log from '192.x.x.2:1224:v3':

exampleuser@%1 !> list
  ID Own                      Name     Status              Owner Information
  %1           192.x.x.2:1224:v3      Fresh                               
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': '' 

""", 'children':{}},


  'start':{
    'name':'start', 'callback':None, 'example':'file [args ...]',
    'summary':"Start an experiment (doesn't upload)", 'cmdgroup': 'extended', 'help_text':"""
start programname [arg1 arg2 ...]

Begins executing a file in the vessel named programname with the given 
arguments.   This program must first be uploaded to the vessel (the 'run'
command does this for the user).

This command will make an educated guess as to what platform your
program is written for (i.e. repyV1 or repyV2).  You can override this
by using 'startv1' or 'startv2', respectively.

Example:
exampleuser@%1 !> upload example.1.1.r2py
exampleuser@%1 !> start example.1.1.r2py
exampleuser@%1 !> show log
Log from '192.x.x.2:1224:v3':
Hello World

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.start_remotefn, 'help_text':'', 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.start_remotefn_arg, 'help_text':'', 'children':{}},
      }},
  }},

  'startv1':{
    'name':'start', 'callback':None, 'example':'file [args ...]',
    'summary':"Start an experiment (doesn't upload) in repyV1", 'cmdgroup': 'extended', 'help_text':"""
start programname [arg1 arg2 ...]

Begins executing a file in the vessel named programname with the given
arguments in repyv1.   This program must first be uploaded to the vessel
(the 'run' command does this for the user).

Example:
exampleuser@%1 !> upload example.1.1.r2py
exampleuser@%1 !> start example.1.1.r2py
exampleuser@%1 !> show log
Log from '192.x.x.2:1224:v3':
Hello World

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.start_remotefn, 'help_text':'', 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.start_remotefn_arg, 'help_text':'', 'children':{}},
      }},
  }},

  'startv2':{
    'name':'start', 'callback':None, 'example':'file [args ...]',
    'summary':"Start an experiment (doesn't upload) in repyV2", 'cmdgroup': 'extended', 'help_text':"""
start programname [arg1 arg2 ...]

Begins executing a file in the vessel named programname with the given
arguments in repyv2.   This program must first be uploaded to the vessel
(the 'run' command does this for the user).

Example:
exampleuser@%1 !> upload example.1.1.r2py
exampleuser@%1 !> start example.1.1.r2py
exampleuser@%1 !> show log
Log from '192.x.x.2:1224:v3':
Hello World

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.start_remotefn, 'help_text':'', 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.start_remotefn_arg, 'help_text':'', 'children':{}},
      }},
  }},

  'startv1':{
    'name':'start', 'callback':None, 'example':'file [args ...]',
    'summary':"Start an experiment (doesn't upload) in repyV1", 'cmdgroup': 'extended', 'help_text':"""
start programname [arg1 arg2 ...]

Begins executing a file in the vessel named programname with the given
arguments in repyv1.   This program must first be uploaded to the vessel
(the 'run' command does this for the user).

Example:
exampleuser@%1 !> upload example.1.1.r2py
exampleuser@%1 !> start example.1.1.r2py
exampleuser@%1 !> show log
Log from '192.x.x.2:1224:v3':
Hello World

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.start_remotefn, 'help_text':'', 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.start_remotefn_arg, 'help_text':'', 'children':{}},
      }},
  }},

  'startv2':{
    'name':'start', 'callback':None, 'example':'file [args ...]',
    'summary':"Start an experiment (doesn't upload) in repyV2", 'cmdgroup': 'extended', 'help_text':"""
start programname [arg1 arg2 ...]

Begins executing a file in the vessel named programname with the given
arguments in repyv2.   This program must first be uploaded to the vessel
(the 'run' command does this for the user).

Example:
exampleuser@%1 !> upload example.1.1.r2py
exampleuser@%1 !> start example.1.1.r2py
exampleuser@%1 !> show log
Log from '192.x.x.2:1224:v3':
Hello World

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.start_remotefn, 'help_text':'', 'children':{
          '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.start_remotefn_arg, 'help_text':'', 'children':{}},
      }},
  }},


  'stop':{
    'name':'stop', 'callback':command_callbacks.stop,
    'summary':'Stop an experiment but leave the log / files', 'help_text':"""
stop

Stops executing the running program in every vessel in the default group.   
The program will halt as though it were killed / interrupted.   The status
for these vessels will become 'Stopped'.

Example:
exampleuser@%1 !> run endless_loop.r2py 
exampleuser@%1 !> list
  ID Own                      Name     Status              Owner Information
  %1             192.x.x.2:1224:v3    Started                               
exampleuser@%1 !> stop
exampleuser@%1 !> list
  ID Own                      Name     Status              Owner Information
  %1             192.x.x.2:1224:v3    Stopped                                

""", 'children':{}},


  'split':{
    'name':'split', 'callback':None, 'example':'resourcefn',
    'summary':'Split another vessel off (requires owner)', 'cmdgroup': 'extended', 'help_text':"""
split resourcefile

Divides the vessels in the default group into two new vessels.   The first
vessel will have the size specified in the resource file.   The second will
have the remaining size, minus the size of the offcut resources.   The new
vessels will be known to the shell. 

You must be the owner of the vessel to use the split command.

Example:
exampleuser@browsegood !> show targets
browsegood (empty)
joingood ['192.x.x.192:1224:v3', '192.x.x.192:1224:v4']
joinfail (empty)
%all ['192.x.x.192:1224:v11']
%3 ['192.x.x.192:1224:v11']
exampleuser@browsegood !> on %3
exampleuser@%3 !> split resources.offcut
192.x.x.192:1224:v11 -> (192.x.x.192:1224:v12, 192.x.x.192:1224:v13)
Added group 'split1' with 1 targets and 'split2' with 1 targets
exampleuser@%3 !> show targets
split2 ['192.x.x.192:1224:v13']
split1 ['192.x.x.192:1224:v12']
browsegood (empty)
joingood ['192.x.x.192:1224:v3', '192.x.x.192:1224:v4']
joinfail (empty)
%5 ['192.x.x.192:1224:v13']
%4 ['192.x.x.192:1224:v12']
%all ['192.x.x.192:1224:v12', '192.x.x.192:1224:v13']

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.split_resourcefn, 'help_text':'', 'children':{}},
  }},


  'join':{
    'name':'join', 'callback':command_callbacks.join,
    'summary':'Join vessels on the same node (requires owner)', 'cmdgroup': 'extended', 'help_text':"""
join

Joins any vessels in the default group that are on the same node.   The 
resulting vessel will have a size equal to the two other vessels plus the
offcut resource amount.  The new vessels will be known to the shell. 

You must be the owner of the vessels to join them.

Example:
exampleuser@browsegood !> join
Traceback (most recent call last):
  File "seash.py", line 222, in command_loop
  File "/home/user/seattle_test/seash_dictionary.py", line 1458, in command_dispatch
  File "/home/user/seattle_test/command_callbacks.py", line 2485, in join
KeyError: 'ownerkey'
exampleuser@browsegood !> update
exampleuser@browsegood !> show owner
192.x.x.192:1224:v3 guest0 pubkey
192.x.x.192:1224:v4 guest0 pubkey
exampleuser@browsegood !> join
192.x.x.192:1224:v11 <- (192.x.x.192:1224:v3, 192.x.x.192:1224:v4)
Added group 'joingood' with 2 targets
exampleuser@browsegood !> show targets
browsegood (empty)
joingood ['192.x.x.192:1224:v3', '192.x.x.192:1224:v4']
joinfail (empty)
%all ['192.x.x.192:1224:v11']
%3 ['192.x.x.192:1224:v11']

""", 'children':{}},


  'exit':{
    'name':'exit', 'callback':command_callbacks.exit,
    'summary':'Exits the shell', 'help_text':"""
exit

Exits seash.
""", 'children':{}},


  'loadstate':{
    'name':'loadstate', 'callback':None, 'example':'fn',
    'summary':'Load encrypted shell state from a file with the keyname', 'cmdgroup': 'extended',
    'help_text':"""
loadstate filename

Loads state information from filename using the default identity to decrypt
the file.   This will restore all identities, keys, groups, etc. from a 
previous seash run.  See also 'set autosave on/off.'

Example:
exampleuser@%1 !> savestate testing_state
exampleuser@%1 !> exit
(restart seash.py)
 !> loadkeys exampleuser
 !> as exampleuser
exampleuser@ !> loadstate testing_state
exampleuser@%1 !>

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.loadstate_filename, 'help_text':'', 'children':{}},
  }},


  'savestate':{
    'name':'savestate', 'callback':None, 'example':'fn',
    'summary':"Save the shell's state information to a file with the keyname", 'cmdgroup': 'extended',
    'help_text':"""
savestate filename

Saves state information into a filename, encrypting the data with the default 
identity's private key.  This can be later used to restore the shell's 
settings, groups, and other information.   See also 'set autosave on/off.'

Example:
exampleuser@%1 !> savestate testing_state
exampleuser@%1 !> exit
(restart seash.py)
 !> loadkeys exampleuser
 !> as exampleuser
exampleuser@ !> loadstate testing_state
exampleuser@%1 !>

""", 'children':{
      '[FILENAME]':{'name':'filename', 'callback':command_callbacks.savestate_filename, 'help_text':'', 'children':{}},
  }},


  'update':{
    'name':'update', 'callback':command_callbacks.update,
    'summary': 'Update information about the vessels', 'cmdgroup': 'extended',
  'help_text':"""
update

Reacquire cached state from the vessels.   This state is used for many of the
'show' commands to prevent every operation from needing to contact all vessels.

Example:
exampleuser@%1 !> show info
192.x.x.2:1224:v3 has no information (try 'update' or 'list')
exampleuser@%1 !> update
exampleuser@%1 !> show info
192.x.x.2:1224:v3 {'nodekey': {'e': 65537L, 'n': 924563171010915569497668794725930165347860823L}, 'version': '0.1t', 'nodename': '192.x.x.2'}

""", 'children':{}},


  'contact':{
    'name':'contact', 'callback':None, 'example':'host:port[:vessel]',
    'summary': 'Communicate with a node explicitly', 'cmdgroup': 'extended',
    'help_text':"""
contact IP:port[:vesselname]

Add the specified vessel to the shell.   This bypasses advertise lookups and
directly contacts the node manager.   If the vesselname argument is omitted,
all vessels that can be controlled by the default identity are added.

Example:
exampleuser@ !> contact 192.x.x.2:1224
Added targets: %1(192.x.x.2:1224:v3)

""", 'children':{
      '[ARGUMENT]':{'name':'args', 'callback':command_callbacks.contact, 'help_text':'', 'children':{}},
  }},
}


# A dictionary to be built to keep track of the different variations of commands users will input
commandvariationdict = {
  'users':['user'],
  'hostname':['hostnames'],
  'location':['locations'],
  'coordinates':['coordinate'],
  'owner':['owners'],
  'timeout':['timeouts'],
  'targets':['target'],
  'uploadrate':['uploadrates'],
  'identities':['identity'],
  'ip':['ips'],
  'keys':['key'],
  'files':['file'],
  'resources':['resource'],
  'offcut':['offcuts'],
  'genkeys':['genkey'],
  'loadkeys':['loadkey'],
  'exit':['quit']
}


##### All methods that adds to the seash command dictionary #####


# Creates a deep copy of the seash dictionary while avoiding any of the 
# commands in the passed list.
# Only works in the first level of the dictionary. Will not search for the 
# existence of the avoided command in deeper levels of the dictionary.
def _deep_copy_main_dict(avoided_cmds_list):

  command_dict_copy = seashcommanddict.copy()

  # For every command pattern found in the passed list,
  # the function will delete it from
  for cmd in avoided_cmds_list:

    if cmd in command_dict_copy:

      del command_dict_copy[cmd]
  

  return command_dict_copy




# Returns a copy of a command's dictionary with an empty dictionary for its children
def _shallow_copy(cmd_dict):
  cmd_dict_copy = cmd_dict.copy()
  cmd_dict_copy['children'] = {}
  return cmd_dict_copy





# Returns the seash command dictionary after making any necessary additions to it

def return_command_dictionary():
  
  # Sets the entire seash command dictionary as 'on target' children except itself
  seashcommanddict['on']['children']['[TARGET]']['children'] = _deep_copy_main_dict(['on'])
  
  # Sets the entire seash command dictionary as 'as keyname' children except itself
  seashcommanddict['as']['children']['[KEYNAME]']['children'] = _deep_copy_main_dict(['as'])

  # Preserve the old children
  old_help_children = seashcommanddict['help']['children'] 
  # Sets the entire seash command dictionary as 'help' children except itself
  seashcommanddict['help']['children'] = _deep_copy_main_dict(['help'])
  
  for child in old_help_children:
    seashcommanddict['help']['children'][child] = old_help_children[child]

  return seashcommanddict


# Returns the summaries of all children commands to the passed in input_dict. 
def get_command_summaries(input_dict):
  inputdict_mark = input_dict 
  seashdict_mark = seashcommanddict
  
  # Go to one level above the last command node, the last node can be a display 
  # group node.
  while (inputdict_mark[inputdict_mark.keys()[0]]['children']):
    current_command = inputdict_mark.keys()[0]
    # Take 1 step deeper into the commanddict
    seashdict_mark = seashdict_mark[current_command]['children']
    inputdict_mark = inputdict_mark[current_command]['children']
  
  # If it isn't a display group node, then it is a command node.
  # We want to print out its contents.  If we don't do this, we'll end up 
  # printing one level higher than what we intended.
  if inputdict_mark[inputdict_mark.keys()[0]]['name'] != 'display_group':
    # This is a known command identified directly
    if inputdict_mark.keys()[0] in seashdict_mark:
      seashdict_mark = seashdict_mark[inputdict_mark.keys()[0]]['children']
    # This could be a user argument, such as a filename or a target.  These 
    # names wouldn't be in the commanddict directly.  Replace it with an 
    # argument 
    else:  
      for command in seashdict_mark.keys():
        if command.startswith('[') and command.endswith(']'):
          seashdict_mark = seashdict_mark[command]['children']

  # Compile the list of all command summaries with the specified name
  # summaries[commandgroup][module] = [(commandstr, summary), (commandstr, summary), ...]
  summaries = {}
  commandkeys = seashdict_mark.keys()
  commandkeys.sort()  # Display the list in alphabetical order

  for command in commandkeys:
    # Skip user arguments
    if command.startswith('[') and command.endswith(']'):
      continue
    
    # If cmdgroup is unspecified, put the command in the default summary list
    if 'cmdgroup' in seashdict_mark[command]:
      cmdgroup = seashdict_mark[command]['cmdgroup']
    else:
      cmdgroup = ''
    # If a summary is not specified, then treat it as having '' as a summary.
    if 'summary' in seashdict_mark[command]:
      summary = seashdict_mark[command]['summary']
    else:
      summary = ''

    # commandstr is the command that is displayed to the user, with examples
    # e.g. "show ip [to file]" 
    commandstr = command[:]
    if 'example' in seashdict_mark[command]:
      commandstr +=' ' + seashdict_mark[commandstr]['example']
    # If this command is part of a module, mark it as such in the summary
    # e.g. "Shows the longitude and latitude of the node. (geoip module)"
    if 'module' in seashdict_mark[command]:
      module = seashdict_mark[command]['module']
    else:
      module = None
    if not cmdgroup in summaries:
      summaries[cmdgroup] = {}
    if module not in summaries[cmdgroup]:
      summaries[cmdgroup][module] = []
    summaries[cmdgroup][module].append((commandstr, summary))
  return summaries

  
def get_string_from_command_summarylist(command_summarylist, command_string):
  # Command string is the list of commands to list up to that point
  # e.g. for help show info, the command_string would be 'help show'.
  # Find longest commandcchildhild key
  longest_command = ''
  for (command, summary) in command_summarylist:
    if len(command) > len(longest_command):
      longest_command = command

  # Default terminals are often 79 chars wide
  max_linelen = 79
  divider = ' -- '
  max_summary_len = max_linelen - len(longest_command + command_string) - len(divider)
  retstring = ''
  # Resulting string looks like this...
  # show files        -- helptext for show files
  # show ip [to file] -- helptext for show ip
  #                   -- continued...
  # show keys         -- helptext for show keys
  # ...
  formatstr = ('%-'+str(len(longest_command + command_string))
      +'s%'+str(len(divider))+'s'
      +'%s\n')
  for (command, summary) in command_summarylist:
    command = command_string + command
    divider = ' -- '
    while summary:
      # Extract the data to print on this line
      show_this_line = summary[:max_summary_len]
      summary = summary[max_summary_len:]
      # At the point that where we split, if we are in the middle of a word, 
      # backtrack to the closest whitespace and split there
      if (not show_this_line[-1].isspace() and  # char before split point
          summary and  # summary could be empty at this point
          not summary[0].isspace()): # char after split point        
        # Take the last word away from this line and put it back into summary
        last_word = show_this_line.split()[-1]
        show_this_line = show_this_line[:-len(last_word)]
        summary = last_word + summary
      retstring += formatstr % (command, divider, show_this_line)
      # Don't show the command/divider after the first line
      command = ''
      divider = ''
      summary = summary.lstrip()
  return retstring


##### User input parsing related methods #####

"""
seash's command parser:
The parser receives the string of commands the user inputted and proceeds
to iterate through seashcommanddict to verify that the each of the command
string corresponds to a key of seash's command dictionary, and each subsequent 
string corresponds to a command key of the current command dictionary's 
dictionary of children. 

At the same time, a sub-dictionary of seashcommanddict is being built that holds 
only the chain of dictionaries that corresponds to the user's input, with the only
difference being that any user-inputted argument will replace the command key of the 
command dictionary associated with it. Thus, instead of '[TARGET]', the key to the 
command dictionary would instead be '%1'. 

Targets and Group name arguments will be checked to see if the name specified
actually exists, and general Argument command dictionaries with no children will
have the corresponding user input string and any string that follows be spliced
into a single string with spaces in between each word.

A ParseError will be raised if the argument given for Target or Group does not
exist, and also if the user inputted command word does not correspond to any of the
command keys of the current dictionaries of dictionaries the iterator is looking
at and there are no argument keys to suggest the inputted word is an user argument.
"""
def parse_command(userinput, display_parsed_result=False):
  userinput = userinput.strip()
  parsed_input = seash_modules.preprocess_input(userinput)

  # If any modules performed some operation on the user's input, print out the 
  # result here...
  if display_parsed_result and userinput != parsed_input:
    print "Parsed as:", parsed_input

  userinput = parsed_input
  userstringlist = userinput.split()


  # Dictionary of dictionaries that gets built corresponding to user input
  input_dict = {}
  # Iterator that builds the input_dict
  input_dict_builder = input_dict

  # The iterator that runs through the command dictionary
  seash_dict_mark = return_command_dictionary()


  result_string_list = []
  # Cycles through the user's input string by string
  while userstringlist:
    user_string = userstringlist.pop(0)
    # Adds the user string directly to the result string by default.  
    # Some nodes (like the [ARGUMENT] node) need to change what is added to the
    # result string, and they can modify it as needed.

    # For example, if a user types in add %1 to mygroup, %1 and mygroup both
    # will trigger the [GROUP] user argument.  We want %1 and mygroup to be
    # inserted into the result string, not [GROUP].  However, we will use
    # [GROUP] to identify the node. 
    result_string_list.append(user_string)

    # First, an initial check to see if user's input matches a specified command word
    for cmd_pattern in seash_dict_mark.iterkeys():

      if user_string == cmd_pattern or (cmd_pattern in commandvariationdict.keys() and user_string in commandvariationdict[cmd_pattern]):

        # Appends a copy to avoid changing the master list of dictionaries
        input_dict_builder[cmd_pattern] = _shallow_copy(seash_dict_mark[cmd_pattern])

        # Moves the input dictionary builder into the next empty children dictionary
        input_dict_builder = input_dict_builder[cmd_pattern]['children']

        # Iterates into the children dictionary of the next level of command that may follow
        seash_dict_mark = seash_dict_mark[cmd_pattern]['children']

        break


    # If the user's input string does not match the any of the command pattern directly,
    # looks through the command's children for the possibility of being an
    # user inputed argument, generally denoted by starting with a '['
    else:
      for cmd_pattern in seash_dict_mark.iterkeys():

        # Checks if the input is listed as a valid target, and appends
        # the command dictionary to the input_dict
        if cmd_pattern.startswith('['):
          if cmd_pattern == '[TARGET]' or cmd_pattern == '[GROUP]':
            
            # Compares input with current list of targets and/or groups
            # Raise exception if not found in list of targets
            if user_string not in seash_global_variables.targets:
              raise seash_exceptions.ParseError("Target does not exist")
          
          # Necessity of checking existence of keynames yet to be determined
          #elif cmd_pattern == '[KEYNAME]':
          #  pass

          # Necessity of verifying existence of file yet to be determined
          #elif cmd_pattern == '[FILENAME]':
          #  pass
          
          # simply appends to input_dict
          elif cmd_pattern == '[ARGUMENT]':

            # If ARGUMENT doesn't have any children, joins the rest of the user's input
            # into a single string
            if not seash_dict_mark[cmd_pattern]['children']:
              # Take off the current token from the result string list, and then
              # add it to the user string list for concatenation
              userstringlist = [result_string_list.pop()] + userstringlist
              user_string = " ".join(userstringlist)
              # We are now done, we must clear the list to indicate that
              userstringlist = []

              # Resets the user_string as arg_string for consistency in rest of code
              result_string_list.append(user_string)
  

          # Appends a copy of the dictionary to avoid changing the master list of command dictionaries
          # Also sets the name of the target specified by the user as the key of the command's dictionary
          # for later use by the command's callback
          input_dict_builder[user_string] = _shallow_copy(seash_dict_mark[cmd_pattern])
          
          # Sets itself as the recently-added command dictionary's children to be ready to
          # assign the next command dictionary associated with the next user's input string
          input_dict_builder = input_dict_builder[user_string]['children']
            
          # sets the next list of command dictionaries
          seash_dict_mark = seash_dict_mark[cmd_pattern]['children']

          break

      else:
        # If the user command started with 'help', this could be the display keyword.
        # We then append a node that will appear only in the input_dict so that the command parser can 
        # retrieve the display keyword.
        # We use this to determine if a user wanted to show commands in a specific group of commands.
        # e.g. help set, help extended, etc.

        # We need to check for a space after 'help', otherwise a command such as
        # 'helpnonexistent' would be treated as valid, even if 'helpnonexistent'
        # is not in the command dictionary.
        if userinput.startswith('help '):
          input_dict_builder[user_string] = {'name': 'display_group', 'children': {}}

        # If the user input doesn't match any of the pattern words and there's no
        # pattern that suggest it may be a user input argument, raise an exception
        # for going outside of seash's command dictionary
        else:    
          raise seash_exceptions.ParseError("Command not understood")



  return input_dict



"""
seash's command dispatcher:
Taking in the input dictionary of dictionaries, input_dict, the dispatcher iterates
through the series of dictionaries and executes the command callback function of
any command dictionaries that has the key 'priority' while keeping reference of the
last callback function of the command dictionary that had one. After completing
the iteration, if the referenced command callback function has not been executed
already from priority status, the dispatcher proceeds to executes the function. 

Each callback function will be passed a copy of the input_dict for access to 
user-inputted arguments and the environment_dict that keeps track of the current 
state of seash.

A DispatchError will be raised if at the end of the iteration there are no valid command
callbacks to be executed.
"""
def command_dispatch(input_dict, environment_dict):
  
  # Iterator through the user command dictionary
  dict_mark = input_dict


  # Sets the command callback method to be executed
  current_callback = None

  
  # Sets the last 'interrupt' command's callback method that was executed.
  # Used to avoid having current_callback execute the same command function again
  interrupt_callback = None


  # First, general check for any command dictionaries with the 'priority' key
  # Execute the callback methods of those commands
  
  while dict_mark.keys():


    # Pulls out the command word, which also serves as the key to the command's dictionary
    # It should be the only key in the key list of the children's dictionary
    command_key = dict_mark.keys()[0]


    # Sets the callback method reference if command's 'callback' isn't set to None
    if dict_mark[command_key]['callback'] is not None:
      current_callback = dict_mark[command_key]['callback']


    # Executes the callback method of all commands that contains the 'priority' key
    if 'priority' in dict_mark[command_key]:
      interrupt_callback = dict_mark[command_key]['callback']
      interrupt_callback(input_dict.copy(), environment_dict)

      # In the case of 'help', breaks out of the dispatch loop to avoid executing any other
      # command's function
      if command_key == 'help':
        break


    # Iterates into the next dictionary of children commands
    dict_mark = dict_mark[command_key]['children']



  # Raises an exception if current_callback is still None
  if current_callback is None:
    raise seash_exceptions.DispatchError("Invalid command. Please check that the command has been inputted correctly.")


  # Executes current_callback's method if it's not the same one as interrupt_callback
  elif not interrupt_callback == current_callback:
    current_callback(input_dict.copy(), environment_dict)



"""
Make sure that preprocessed command line input is printed to the screen
"""

import seash
import seash_dictionary
import seash_modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)


#pragma out Enabled modules:

#pragma out loadkeys guest0
#pragma out as guest0

seash.command_loop([
  'enable variables',
  'set username guest0',
  'loadkeys $username',
  'as $username$',
  'disable variables'])


"""
Ensure that modules that have conflicting commands do not get imported.
It should also display a readable error message indicating where the conflicting
command was found.
"""

import seash_dictionary
import seash_modules
#pragma out Enabled modules: modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)

#pragma out Module cannot be imported due to the following conflicting command:
#pragma out show (default)

import seash
seash.command_loop([
  'enable conflict'
])


"""
<Filename>
  factoids/__init__.py

<Purpose>
  Used to print saesh factoids.
  
  It implements the following command:  
    show factoids [number of factoids]/all

"""

import seash_exceptions

import random
import os


# List which will contain all factoids after fatching from a file.
factoids = []


def initialize():
  """
  <Purpose>
    Used to print random seash factoid when user runs seash.
    
  <Arguments>
    None
    
  <Side Effects>
    Prints random factoid onto the screen.
    
  <Exceptions>
    UserError: Error during generating path to "factoid.txt" file or
               Error while opening, reading or closing "factoid.txt" file.
    
  <Return>
    None
  """
  
  # Global 'factoids' list will be used to store factoids, fetched from a file.
  global factoids
  
  # Path to "factoid.txt" file is created.
  try:
    current_path = os.getcwd()
    file_path = os.path.join(current_path, "modules", "factoids", "factoid.txt")
  except OSError, error:
    raise seash_exceptions.InitializeError("Error during initializing factoids module: '" + str(error) + "'.")

  # We have to fatch list of factoids from "factoid.txt" file.
  try:
    file_object = open(file_path, 'r')
    factoids_temp = file_object.readlines()
    file_object.close()
  except IOError, error:
    raise seash_exceptions.InitializeError("Error during initializing factoids module: '" + str(error) + "'.")
  
  # Newline characters in a list, read from a file are removed.
  for factoid in factoids_temp:
    factoids.append(factoid.strip('\n'))
  
  # A random factoid is printed every time user runs seash.
  print random.choice(factoids)+"\n"


def cleanup():
  """
  <Purpose>
    Used to clean 'factoids' list when this module is going to be disabled.
    
  <Arguments>
    None
    
  <Side Effects>
    None
    
  <Exceptions>
    None
    
  <Return>
    None
  """
  
  # Data from a global 'factoids' list will be removed.
  global factoids
  factoids = []


def print_factoids(input_dict, environment_dict):
  """
  <Purpose>
    Used to print seash factoids when user uses 'show factoids'
    command.
    
  <Arguments>
    input_dict: Input dictionary generated by seash_dictionary.parse_command().
    environment_dict: Dictionary describing the current seash environment.
    
    For more information, see command_callbacks.py's module docstring.
    
  <Side Effects>
    Prints factoids onto the screen.
    
  <Exceptions>
    UserError: If user does not type appropriate command.
    ValueError: If user does not provide valid input (integer).
    
  <Return>
    None
  """
  
  # User will insert an argument regarding how many factoids should be printed.
  # We have to find what is user argument.
  # User can type any positive number or he can type 'all' to see all factoids.
  dict_mark = input_dict
  try:
    command = dict_mark.keys()[0]
    while dict_mark[command]['name'] != 'args':
      dict_mark = dict_mark[command]['children']
      command = dict_mark.keys()[0]
    args = command
  except IndexError:
    raise seash_exceptions.UserError("\nError, Syntax of the command is: show factoids [number of factoids]/all \n")
  
  # User decided to print all factoids
  if args == 'all':
    print
    for factoid in factoids:
      print factoid
    print
    return
  
  # User can not insert other than integer number.
  try:
    no_of_factoids = int(args)
  except ValueError:
    raise seash_exceptions.UserError("\nYou have to enter number only.\n")
  
  # If number of factoids decided by user is greater than total number of
  # available factoids than whole factoids list is printed.
  if (no_of_factoids > (len(factoids))):
    print "\nWe have only %d factoids. Here is the list of factoids:" % (len(factoids))
    no_of_factoids = len(factoids)
  elif (no_of_factoids <= 0):
    raise seash_exceptions.UserError("\nYou have to enter positive number only.\n")
  
  # 'factoids' list will be shuffled every time for printing random factoids.
  random.shuffle(factoids)
  
  # Factoids will be printed.
  for factoid in factoids[:no_of_factoids]:
    print factoid
  print
  



command_dict = {
  'show': {'name':'show', 'callback': None, 'children':{
      'factoids':{'name':'factoids', 'callback': print_factoids,
                  'summary': "Displays available seash factoids.",
                  'help_text': '','children':{
          '[ARGUMENT]':{'name':'args', 'callback': None, 'children':{}}
    }},}}
}



help_text = """
Factoids Module

This module includes command that prints seash factoids.
'show factoids [number of factoids]/all' is used to print
available seash factoids.
You can type 'show factoids [number of factoids]' to print
that much number of factoids.
You can type 'show factoids all' to see all available factoids.
"""


# This is where the module importer loads the module from.
moduledata = {
  'command_dict': command_dict,
  'help_text': help_text,
  'url': None,
  'initialize': initialize,
  'cleanup': cleanup
}


"""
<Filename>
  geoip_module.py

<Purpose>
  This is the command dictionary for location-related services for seash.
  For more information on the command dictionary, see the documentation for
  seash_importer.py.
  
  It implements the following commands:  
    show location
    show coordinates

"""

import seash_global_variables
import seash_exceptions
from repyportability import add_dy_support
add_dy_support(locals())
dy_import_module_symbols("geoip_client.r2py")

dy_import_module_symbols("geoip_client.r2py")



# show location        -- Display location information about the nodes
def show_location(input_dict, environment_dict):

  if not environment_dict['currenttarget']:
    raise seash_exceptions.UserError("Error, command requires a target")

  geoip_init_client()

  # we should only visit a node once...
  printedIPlist = []

  for longname in seash_global_variables.targets[environment_dict['currenttarget']]:
    thisnodeIP = seash_global_variables.vesselinfo[longname]['IP']

    # if we haven't visited this node
    if thisnodeIP not in printedIPlist:
      printedIPlist.append(thisnodeIP)

      try:
        # The GeoIP server doesn't understand hostnames. thisnodeIP
        # will be a hostname if it is a NAT node and we're using
        # Affixes.
        node_ip = gethostbyname(thisnodeIP)
        location_dict = geoip_record_by_addr(node_ip)
      # Name resolution and geoip lookups could fail, in which case
      # we just don't know where the node is.
      except:
        location_dict = None

      if location_dict:
        print str(seash_global_variables.vesselinfo[longname]['ID'])+'('+str(thisnodeIP)+'): '+geoip_location_str(location_dict)
      else:
        print str(seash_global_variables.vesselinfo[longname]['ID'])+'('+str(thisnodeIP)+'): Location unknown'


#show coordinates -- Display the latitude & longitude of the nodes
def show_coordinates(input_dict, environment_dict):

  if not environment_dict['currenttarget']:
    raise seash_exceptions.UserError("Error, command requires a target")

  geoip_init_client()

  # we should only visit a node once...
  printedIPlist = []

  for longname in seash_global_variables.targets[environment_dict['currenttarget']]:
    thisnodeIP = seash_global_variables.vesselinfo[longname]['IP']

    # if we haven't visited this node
    if thisnodeIP not in printedIPlist:
      printedIPlist.append(thisnodeIP)

      try:
        # The GeoIP server doesn't understand hostnames. thisnodeIP
        # will be a hostname if it is a NAT node and we're using
        # Affixes.
        node_ip = gethostbyname(thisnodeIP)
        location_dict = geoip_record_by_addr(node_ip)
      # Name resolution and geoip lookups could fail, in which case
      # we just don't know where the node is.
      except:
        location_dict = None

      if location_dict:
        print str(seash_global_variables.vesselinfo[longname]['ID'])+'('+str(thisnodeIP)+'): ' + str(location_dict['latitude']) + ", " + str(location_dict['longitude'])

      else:
        print str(seash_global_variables.vesselinfo[longname]['ID'])+'('+str(thisnodeIP)+'): Location unknown'


SHOW_LOCATION_HELPTEXT = """
show location

Uses a geo-IP location service to return information about the position of the
nodes in the current group.

Example:
exampleuser@browsegood !> show ip
192.x.x.2
193.x.x.42
219.x.x.62
exampleuser@browsegood !> show location
%1(192.x.x.2): Location unknown
%3(193.x.x.42): Cesson-svign, France
%4(219.x.x.62): Beijing, China

"""

SHOW_COORDINATES_HELPTEXT = """
show coordinates

Uses a geo-IP location service to get approximate latitude and longitude
information about nodes in the current group.

Example:
exampleuser@browsegood !> show location
%1(192.x.x.2): Location unknown
%3(193.x.x.42): Cesson-svign, France
%4(219.x.x.62): Beijing, China
exampleuser@browsegood !> show coordinates
%1(192.x.x.2): Location unknown
%3(193.x.x.42): 48.1167, 1.6167
%4(219.x.x.62): 39.9289, 116.3883

"""

command_dict = {
  'show':{
    'children': {
      'location':{
      'name':'location',
      'callback': show_location,
      'summary': "Display location information (countries) for the nodes",
      'help_text':SHOW_LOCATION_HELPTEXT,
      'children': {}},
    'coordinates':{
      'name':'coordinates',
      'callback': show_coordinates,
      'summary':'Display the latitude & longitude of the node',
      'help_text':SHOW_COORDINATES_HELPTEXT,
      'children': {}}
    }}
}


help_text = """
GeoIP Module

This module includes commands that provide information regarding vessels' 
geographical locations.  To get started using this module, acquire several 
vessels through the Seattle Clearinghouse, use the 'browse' command, and then 
in any group, run either 'show location' or 'show coordinates'.
"""


# This is where the module importer loads the module from
moduledata = {
  'command_dict': command_dict,
  'help_text': help_text,
  'url': None,
}



"""
Make sure that disabling a disabled module fails
"""
import seash
import seash_exceptions

# We need to do module manipulation
import seash_dictionary
import seash_modules
#pragma out Enabled modules: modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)

seash.command_loop(["enable geoip"])

# This should fail.  We can't use the built in UTF pragma because we need to 
# disable the module before we finish this test.
try:
  seash.command_loop(["enable geoip"])
except seash_exceptions.UserError, e:
  if "Module is already enabled." not in str(e):
    raise

seash.command_loop(["disable geoip"])


"""
<Program Name>
  seash_importer.py

<Purpose>
  This is the module importer for seash. It allows developers to easily extend
  the functionality of seash without having to modify core seash files.

  The importer will expect a folder containing a file called __init__.py.
  This will be referenced here as the commanddict file. For every module, there
  must be exactly one commanddict file. Simply add this folder into the modules/
  folder, and seash will import it automatically.

  Inside the commanddict file, there should be a variable in the module scope
  named command_dict. You can define new commands in the same manner as
  described in seash_dictionary.py.

  Note, you cannot override existing commands. Attempting to do so will result
  in an error.

"""
import warnings
import seash_exceptions
import os
import sys
from copy import deepcopy

# All seash modules are subdirectories under this directory.
MODULES_FOLDER_PATH = os.path.abspath('modules')

# Stores all the imported commanddicts
module_data = {}

def import_module(modulefn):
  """
  <Purpose>
    Imports a seash module with the specified modulename.
    The seash module is treated as a python package  
    
  <Arguments>
    modulefn:
      The name of the main modules file.
      
  <Side Effects>
    The commands found in modulename, alongside the helptext for the module
    will be imported and returned.
    
  <Exceptions>
    ImportError
    
  <Return>
    A dictionary containing the command_dict and the helptext.

    An example: {
      'command_dict': {'command1': ...,'command2':...},
      'help_text': 'This is the module helpstring'
    }

  """
  # We can't import python modules by specifying the full path to the module
  # Temporarily add the module path to the pythonpath
  sys.path = [MODULES_FOLDER_PATH] + sys.path
  moduleobj = __import__(modulefn)
  try:
    _attach_module_identifier(moduleobj.moduledata['command_dict'], modulefn)
    return moduleobj.moduledata
  except (NameError, KeyError):
    raise seash_exceptions.ModuleImportError("Module '" + modulefn + "' is not well defined")
  finally:
    # Remove the module path from the pythonpath because we don't need it anymore
    sys.path = sys.path[1:]


def import_all_modules():
  """
  <Purpose>
    Imports all modules within the modules folder.  This should only be called once 
    throughout the entire execution of seash.
      
  <Side Effects>
    Modules that don't have collisions will have their commanddicts and 
    helptexts loaded and returned.
    
  <Exceptions>
    ImportError: There is an existing module with the same name already imported.
      
  <Return>
    The seashcommanddict that contains the imported commands on top of the 
    passed in commanddict.

  """
  for module_folder in get_installed_modules(): 
    try:
      if module_folder in module_data:
        raise seash_exceptions.ModuleImportError("Module already imported")
      module_data[module_folder] = import_module(module_folder)
    except seash_exceptions.ModuleImportError, e:
      print str(e)




def ensure_no_conflicts_in_commanddicts(originaldict, comparedict):
  """
  <Purpose>
    Recursively compares two commanddicts to see if they have conflicting commands.
    
  <Arguments>
    originaldict: A commanddict to compare.
    comparedict: A commanddict to compare.
    
  <Side Effects>
    None
    
  <Exceptions>
    ModuleConflictError - A command was conflicting.  
        The error detail is the problematic command.
    
  <Returns>
    None
  """
  
  """
  Child nodes are identical if they all of the following are identical: 
    helptext/callback/summary.
     
  There are 3 cases we have to worry about.
  > Shared child node.
    > Child nodes are identical.  Check grandchildren.
    > Only one is defined.  Check grandchildren.
    > Both child nodes are defined and are not identical.  Reject.
  > Node is not shared.  Accept. 
  """
  
  for child in comparedict.keys():
    # Node not shared.
    if child not in originaldict:
      continue
    
    # Shared node
    comparechild_defined = is_commanddictnode_defined(comparedict[child])
    originalchild_defined = is_commanddictnode_defined(originaldict[child])
    
    # Only one is defined, or;
    # both are defined and they are identical
    if ((comparechild_defined ^ originalchild_defined) or
        (comparechild_defined and originalchild_defined and 
         _are_cmd_nodes_same(originaldict[child], comparedict[child]))):
      try:
        ensure_no_conflicts_in_commanddicts(comparedict[child]['children'], originaldict[child]['children'])
      except seash_exceptions.ModuleConflictError, e:
        # Reconstruct the full command recursively
        raise seash_exceptions.ModuleConflictError(child + " " + str(e) + " ("+module_name+")")
      continue
    
    # Not identical.  Conflict found.
    # Also include which module the conflicting module was found from.
    if 'module' in originaldict[child]:
        module_name = originaldict['module'][child]      
    else:
      module_name = "default"
    raise seash_exceptions.ModuleConflictError(child + ' ('+module_name+')')


def is_commanddictnode_defined(node):
  """
  A child node is defined if it has either a helptext/callback/summary.
  If a node's callback is None it can still be undefined. 
  """
  return (('callback' in node and not node['callback'] is None) or
           'help_text' in node or
           'summary' in node)


def _are_cmd_nodes_same(node1, node2):
  """ 
  Checks to see if two cmddnodes are the same.
  Two cmdnodes are defined to be the same if they have the same callbacks/
  helptexts/summaries. 
  """

  # Everything in node1 should be in node2
  for propertytype in node1:
    if (not propertytype in node2 or
        node1[propertytype] != node2[propertytype]):
      return False
  return True


def are_cmddicts_same(dict1, dict2):
  """ 
  Checks to see if two cmddicts are the same.
  Two cmddicts are defined to be the same if they have the same callbacks/
  helptexts/children/summaries for all nodes. 
  """
  
  # If the set of all keys are not the same, they must not be the same.
  if set(dict1.keys()) != set(dict2.keys()):
    return False
  
  # Everything in dict1 should be in dict2
  for key in dict1:
    # Check everything except children;  Check for children recursively
    for propertytype in dict1[key]:
      if (not propertytype in dict2[key] or
          dict1[key][propertytype] != dict2[key][propertytype]):
        return False
      
    # Check children
    if not are_cmddicts_same(dict1[key]['children'], dict2[key]['children']):
      return False
      
  return True


def merge_commanddict_recursive(originaldict, mergedict):
  """
  <Purpose> 
    Recursively merge mergedict into originaldict.
    We assume that there are no conflicting modules here.  
    Be sure to check that there aren't any collisions!
    
  <Arguments>
    originaldict: The commanddict to merge to.
    mergedict: The commanddict to merge from.
  
  <Side Effects>
    Originaldict will contain all command entries in mergedict.
    
  <Exceptions>
    There shouldn't be any...
    
  <Return>
    None
  """
  
  """
  Every command in the mergedict should be placed into the original.  
  We do not handle the case where a shared node is defined on both sides.
  That check is done by ensure_no_conflicts_in_commanddict().
  We make a deep copy of mergedict to make the deletion case easier.
  """
  for commandnode in mergedict:
    # Trivial case
    if commandnode not in originaldict:
      originaldict[commandnode] = deepcopy(mergedict[commandnode])
    
    else:
      # Shared node exists in original but is not defined
      # Replace properties if they exist, and then merge over the children.
      if not is_commanddictnode_defined(originaldict[commandnode]):
        for entry in mergedict[commandnode]:
          if not entry in ['children', 'module']:
            originaldict[commandnode][entry] = mergedict[commandnode][entry]
      
      merge_commanddict_recursive(originaldict[commandnode]['children'], mergedict[commandnode]['children'])
    
    
      

def merge_commanddict(originaldict, mergedict):
  """
  <Purpose>
    Merges two command dictionaries.  This is used to add commands to the main
    seash commanddict.  Remember to perform the same call on under the help node 
    so that the help command works as expected.
    
    e.g.
      merge_commanddict(seashcommanddict, mycommanddict)
      merge_commanddict(seashcommanddict['help']['children'], mycommanddict) 
    
  <Arguments>
    cmddict_original: The commanddict to merge to.
    cmddict_merge: The commanddict from which to merge commands from.
    
  <Side Effects>
    All commands from cmddict_merge will be merged into cmddict_original.
    This assumes that all commands are not conflicting.  If they are, an exception is raised.
    
  <Exceptions>
    ModuleConflictError: A conflicting command was found.
  
  <Returns>
    None
  """
  
  ensure_no_conflicts_in_commanddicts(originaldict, mergedict)
  merge_commanddict_recursive(originaldict, mergedict)


def remove_commanddict(originaldict, removedict):
  """
  <Purpose>
    Removes all commands found in a command dictionary from another command 
    dictionary. Remember to perform the same call on under the help node 
    so that the help command works as expected.
    
    e.g.
      remove_commanddict(seashcommanddict, mycommanddict)
      remove_commanddict(seashcommanddict['help']['children'], mycommanddict) 
    
  <Arguments>
    originaldict: The commanddict to remove from.
    removedict: The commanddict containing the commands to remove.
    
  <Side Effects>
    All commands in cmddict_merge will be removed from originaldict.
    A node will not be removed while there are children under that node.
    However, if a parent node is undefined and the last defined child is removed,
    that parent node will be removed as well.
    
  <Exceptions>
    None
  
  <Returns>
    None
  """
  
  for child in removedict:
    if child in originaldict:
      # Recursively remove all children specified
      remove_commanddict(originaldict[child]['children'], removedict[child]['children'])
      # Remove the definition as well if it is defined in removedict
      if is_commanddictnode_defined(removedict[child]):
        # Remove everything except for children.  We remove those recursively.
        for propertytype in removedict[child]:
          # Not all properties (i.e. module) will be defined in the original 
          # dictionary.  We may raise an exception when trying to delete one
          # such property.
          if (propertytype != 'children' and
              propertytype in originaldict[child]):
            del originaldict[child][propertytype]
      # Remove this node if this node is no longer defined, and if there are no 
      # remaining child nodes.
      if not (is_commanddictnode_defined(originaldict[child]) or originaldict[child]['children']):
        del originaldict[child]


def enable(commanddict, modulename):
  """
  <Purpose>
    Enables a module and imports its commands into the seash commanddict.

  <Arguments>
    modulename: The module to import.

  <Side Effects>
    All commands inside the specified module will be inserted into the seash
    commanddict if possible.

    The file modulename.disabled will be removed from /modules/ indicating that
    this module has been enabled.

  <Exceptions>
    Exceptions raised by merge_commanddict()

  <Returns>
    None
  """
  # Is this an installed module?
  if not modulename in module_data:
    raise seash_exceptions.UserError("Error, module '"+modulename+"' is not installed")

  if _is_module_enabled(modulename):
    raise seash_exceptions.UserError("Module is already enabled.")

  merge_commanddict(commanddict, module_data[modulename]['command_dict'])

  try:
    # We mark this module as enabled by deleting the modulename.disabled file
    os.remove(MODULES_FOLDER_PATH + os.sep + modulename + ".disabled") 
  except OSError, e:
    # If the file was deleted before we were able to delete it, it should not
    # be a problem.
    if not "cannot find the file" in str(e):
      raise
  
  try:
    initialize(modulename)
  except seash_exceptions.InitializeError, e:
    raise seash_exceptions.InitializeError(e)


def disable(commanddict, modulename):
  """
  <Purpose>
    Disables a module and removes its commands from the seash commanddict.

  <Arguments>
    modulename: The module to disable.

  <Side Effects>
    All commands inside the specified module will be removed from the seash
    commanddict.

    A file (modulename.disabled) will be created under /modules/ indicating that
    this module has been disabled.

  <Exceptions>
    Exceptions raised by merge_commanddict()

  <Returns>
    None
  """
  # Is this an installed module?
  if not modulename in module_data:
    raise seash_exceptions.UserError("Error, module '"+modulename+"' is not installed")
  
  # Is this module enabled?
  if not _is_module_enabled(modulename):
    raise seash_exceptions.UserError("Module is not enabled.")

  remove_commanddict(commanddict, module_data[modulename]['command_dict'])
  cleanup(modulename)

  # We mark this module as disabled by adding a modulename.disabled file.
  open(MODULES_FOLDER_PATH + os.sep + modulename + ".disabled", 'w')


def _is_module_enabled(modulename):
  """
  A module is enabled if there is not a "modulename.disabled" file under the /modules
  folder.
  """
  disabled_filename = modulename + '.disabled'
  return not disabled_filename in os.listdir('./modules/')


def get_enabled_modules():
  """
  <Purpose>
    Returns all enabled modules.
  <Arguments>
    None
  <Side Effects>
    None
  <Exceptions>
    None
  <Return>
    The list of all enabled modules.
  """
  enabled = []
  directory_contents = os.listdir(MODULES_FOLDER_PATH)
  for fname in get_installed_modules():
    if not fname+'.disabled' in directory_contents:
      enabled.append(fname)
  return enabled


def enable_modules_from_last_session(seashcommanddict):
  """
  Enable every module that isn't marked as disabled in the modules folder.
  This function is meant to be called when seash is initializing and nowhere
  else.  A module is marked as disabled when there is a modulename.disabled 
  file.
  """
  successfully_enabled_modules = []
  modules_to_enable = get_enabled_modules()
  for modulename in modules_to_enable:
    # There are no bad side effects to seash's state when we do this
    # The only thing that should happen is that the modulename.disabled file
    # gets created (temporarily)
    disable(seashcommanddict, modulename)
    try:
      enable(seashcommanddict, modulename)
      successfully_enabled_modules.append(modulename)
    except seash_exceptions.ModuleConflictError, e:
      print "Failed to enable the '"+modulename+"' module due to the following conflicting command:"
      print str(e)

      # We mark this module as disabled by adding a modulename.disabled file.
      open(MODULES_FOLDER_PATH + os.sep + modulename + ".disabled", 'w')
    except seash_exceptions.InitializeError, e:
      print "Failed to enable the '"+modulename+"' module."
      disable(seashcommanddict, modulename)
  successfully_enabled_modules.sort()
  
  print 'Enabled modules:', ', '.join(successfully_enabled_modules), '\n'


def tab_complete(input_list):
  """
  <Purpose>
    Gets the list of all valid tab-complete strings from all enabled modules.
  <Arguments>
    input_list: The list of words the user entered.
  <Side Effects>
    None
  <Exceptions>
    None
  <Returns>
    A list of valid tab-complete strings
  """
  commands = []
  for module in get_enabled_modules():
    if 'tab_completer' in module_data[module]:
      commands += module_data[module]['tab_completer'](input_list)
  return commands


def preprocess_input(userinput):
  """
  <Purpose>
    Preprocess the raw command line input string.

  <Arguments>
    The raw command line input string.  We assume it is pre-stripped.

  <Side Effects>
    The string will be processed by each module that has a defined preprocessor.

  <Exceptions>
    None

  <Returns>
    The preprocessed string.
  """
  for module in get_enabled_modules():
    # Not every module has a preprocessor...
    if 'input_preprocessor' in module_data[module]:
      userinput = module_data[module]['input_preprocessor'](userinput)
  return userinput


def _attach_module_identifier(command_dict, modulefn):
  """
  Attaches a 'module': modulename entry to each node in the dictionary.
  This is used by the help printer so that the user can tell if a command was
  included by default or via a module. 
  """
  for command in command_dict:
    command_dict[command]['module'] = modulefn
    _attach_module_identifier(command_dict[command]['children'], modulefn)



def get_installed_modules():
  modules = []
  for folder in os.listdir(MODULES_FOLDER_PATH): 
    if os.path.isfile(MODULES_FOLDER_PATH + os.sep + folder):
      continue

    # Needed to differentiate modules from other folders that are not part of the 
    # module system that are system-generated, like .svn/
    if not '__init__.py' in os.listdir(MODULES_FOLDER_PATH + os.sep + folder):
      continue
    
    modules.append(folder)
  return modules


def initialize(modulename):
  """
  <Purpose>
    Performs initialization steps for the module.

  <Arguments>
    None

  <Side Effects>
    None

  <Exceptions>
    None

  <Returns>
    None
  """
  
  # Not every module needs initialization...
  if 'initialize' in module_data[modulename]:
    module_data[modulename]['initialize']()
  

def cleanup(modulename):
  """
  <Purpose>
    Performs cleanup steps for the module.

  <Arguments>
    None

  <Side Effects>
    None

  <Exceptions>
    None

  <Returns>
    None
  """
  
  # Not every module needs cleanup...
  if 'cleanup' in module_data[modulename]:
    module_data[modulename]['cleanup']()


def _ensure_module_folder_exists():
  """ 
  Checks to see if the module folder exists. If it does not, create it.
  If there is an existing file with the same name, we raise a RuntimeError.
  """
  if not os.path.isdir(MODULES_FOLDER_PATH):
    try:
      os.mkdir(MODULES_FOLDER_PATH)
    except OSError, e:
      if "file already exists" in str(e):
        raise RuntimeError("Could not create modules folder: file exists with the same name")


_ensure_module_folder_exists()
import_all_modules()


"""
Sets up the local node manager processes for certain tests
"""

import subprocess
import sys
import harshexit
import os
import glob
import shutil

initproc = subprocess.Popen([sys.executable, "nminit_test.py"])
initproc.wait()

nmproc = subprocess.Popen([sys.executable, "nmmain.py", "--test-mode"], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

# without closing stdin, stdout, stderr, nmmain.py won't execute on XP
nmproc.stdin.close()
nmproc.stdout.close()
nmproc.stderr.close()


try:
  # We'll want to kill this once we are told to do so... (sys.stdin.close() is
  # the signal)
  sys.stdin.read()
except KeyboardInterrupt:
  # the user interrupted us, let's clean up!
  print
  print 'Cleaning up the subprocess'

# We need the PID.   Since the node manager daemonizes, this isn't the pid of
# nmproc.   We can get the PID from the log
nodemanagerlogfo = open('v2'+os.sep+'nodemanager.old')

firstline = nodemanagerlogfo.readline()

# an entry looks like this:
# 1292537407.82:PID-27493:[INFO]: Running nodemanager in test mode
# let's get the PID out as a number...
pidportion = firstline.split(':')[1]
assert(pidportion.startswith('PID-'))
nmpid = int(pidportion[4:])

# let's terminate the node manager...
harshexit.portablekill(nmpid)

# remove the directories and files created by nminit_test.py
# first, let's clean up any existing directory data...
for vesseldirectoryname in glob.glob('v[0-9]*'):
  if os.path.isdir(vesseldirectoryname):
    print 'Removing:',vesseldirectoryname
    shutil.rmtree(vesseldirectoryname)

# remove resource files created by nminit_test.py
for resourcefilename in glob.glob('resource.v*[0-9]'):
  if os.path.isfile(resourcefilename):
    os.remove(resourcefilename)

# remove example repy files created from downloading from node
for repyfiles in glob.glob('example.1.1.r2py.*'):
  if os.path.isfile(repyfiles):
    os.remove(repyfiles)

# remove other files created by nminit_test.py
os.remove("resources.offcut")
os.remove("vesseldict")



# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2003-2006 Gary Bishop.
#       Copyright (C) 2006  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
import System
from common import validkey, KeyPress, make_KeyPress_from_keydescr

c32 = System.ConsoleKey
Shift = System.ConsoleModifiers.Shift
Control = System.ConsoleModifiers.Control
Alt = System.ConsoleModifiers.Alt
# table for translating virtual keys to X windows key symbols
code2sym_map = {#c32.CANCEL:    u'Cancel',
                c32.Backspace:  u'BackSpace',
                c32.Tab:        u'Tab',
                c32.Clear:      u'Clear',
                c32.Enter:      u'Return',
#                c32.Shift:     u'Shift_L',
#                c32.Control:   u'Control_L',
#                c32.Menu:      u'Alt_L',
                c32.Pause:      u'Pause',
#                c32.Capital:   u'Caps_Lock',
                c32.Escape:     u'Escape',
#                c32.Space:     u'space',
                c32.PageUp:     u'Prior',
                c32.PageDown:   u'Next',
                c32.End:        u'End',
                c32.Home:       u'Home',
                c32.LeftArrow:  u'Left',
                c32.UpArrow:    u'Up',
                c32.RightArrow: u'Right',
                c32.DownArrow:  u'Down',
                c32.Select:     u'Select',
                c32.Print:      u'Print',
                c32.Execute:    u'Execute',
#                c32.Snapshot:  u'Snapshot',
                c32.Insert:     u'Insert',
                c32.Delete:     u'Delete',
                c32.Help:       u'Help',
                c32.F1:         u'F1',
                c32.F2:         u'F2',
                c32.F3:         u'F3',
                c32.F4:         u'F4',
                c32.F5:         u'F5',
                c32.F6:         u'F6',
                c32.F7:         u'F7',
                c32.F8:         u'F8',
                c32.F9:         u'F9',
                c32.F10:        u'F10',
                c32.F11:        u'F11',
                c32.F12:        u'F12',
                c32.F13:        u'F13',
                c32.F14:        u'F14',
                c32.F15:        u'F15',
                c32.F16:        u'F16',
                c32.F17:        u'F17',
                c32.F18:        u'F18',
                c32.F19:        u'F19',
                c32.F20:        u'F20',
                c32.F21:        u'F21',
                c32.F22:        u'F22',
                c32.F23:        u'F23',
                c32.F24:        u'F24',
#                c32.Numlock:    u'Num_Lock,',
#                c32.Scroll:     u'Scroll_Lock',
#                c32.Apps:       u'VK_APPS',
#                c32.ProcesskeY: u'VK_PROCESSKEY',
#                c32.Attn:       u'VK_ATTN',
#                c32.Crsel:      u'VK_CRSEL',
#                c32.Exsel:      u'VK_EXSEL',
#                c32.Ereof:      u'VK_EREOF',
#                c32.Play:       u'VK_PLAY',
#                c32.Zoom:       u'VK_ZOOM',
#                c32.Noname:     u'VK_NONAME',
#                c32.Pa1:        u'VK_PA1',
                c32.OemClear:  u'VK_OEM_CLEAR',
                c32.NumPad0:    u'NUMPAD0',
                c32.NumPad1:    u'NUMPAD1',
                c32.NumPad2:    u'NUMPAD2',
                c32.NumPad3:    u'NUMPAD3',
                c32.NumPad4:    u'NUMPAD4',
                c32.NumPad5:    u'NUMPAD5',
                c32.NumPad6:    u'NUMPAD6',
                c32.NumPad7:    u'NUMPAD7',
                c32.NumPad8:    u'NUMPAD8',
                c32.NumPad9:    u'NUMPAD9',
                c32.Divide:     u'Divide',
                c32.Multiply:   u'Multiply',
                c32.Add:        u'Add',
                c32.Subtract:   u'Subtract',
                c32.Decimal:    u'VK_DECIMAL'
               }

# function to handle the mapping
def make_keysym(keycode):
    try:
        sym = code2sym_map[keycode]
    except KeyError:
        sym = u''
    return sym

sym2code_map = {}
for code,sym in code2sym_map.iteritems():
    sym2code_map[sym.lower()] = code

def key_text_to_keyinfo(keytext):
    u'''Convert a GNU readline style textual description of a key to keycode with modifiers'''
    if keytext.startswith('"'): # "
        return keyseq_to_keyinfo(keytext[1:-1])
    else:
        return keyname_to_keyinfo(keytext)


def char_to_keyinfo(char, control=False, meta=False, shift=False):
    vk = (ord(char))
    if vk & 0xffff == 0xffff:
        print u'VkKeyScan("%s") = %x' % (char, vk)
        raise ValueError, u'bad key'
    if vk & 0x100:
        shift = True
    if vk & 0x200:
        control = True
    if vk & 0x400:
        meta = True
    return (control, meta, shift, vk & 0xff)

def keyname_to_keyinfo(keyname):
    control = False
    meta = False
    shift = False

    while 1:
        lkeyname = keyname.lower()
        if lkeyname.startswith(u'control-'):
            control = True
            keyname = keyname[8:]
        elif lkeyname.startswith(u'ctrl-'):
            control = True
            keyname = keyname[5:]
        elif lkeyname.startswith(u'meta-'):
            meta = True
            keyname = keyname[5:]
        elif lkeyname.startswith(u'alt-'):
            meta = True
            keyname = keyname[4:]
        elif lkeyname.startswith(u'shift-'):
            shift = True
            keyname = keyname[6:]
        else:
            if len(keyname) > 1:
                return (control, meta, shift, sym2code_map.get(keyname.lower(),u" "))
            else:
                return char_to_keyinfo(keyname, control, meta, shift)

def keyseq_to_keyinfo(keyseq):
    res = []
    control = False
    meta = False
    shift = False

    while 1:
        if keyseq.startswith(u'\\C-'):
            control = True
            keyseq = keyseq[3:]
        elif keyseq.startswith(u'\\M-'):
            meta = True
            keyseq = keyseq[3:]
        elif keyseq.startswith(u'\\e'):
            res.append(char_to_keyinfo(u'\033', control, meta, shift))
            control = meta = shift = False
            keyseq = keyseq[2:]
        elif len(keyseq) >= 1:
            res.append(char_to_keyinfo(keyseq[0], control, meta, shift))
            control = meta = shift = False
            keyseq = keyseq[1:]
        else:
            return res[0]

def make_keyinfo(keycode, state):
    control = False
    meta  =False
    shift = False
    return (control, meta, shift, keycode)


def make_KeyPress(char, state, keycode):

    shift = bool(int(state) & int(Shift))
    control = bool(int(state) & int(Control))
    meta = bool(int(state) & int(Alt))
    keyname = code2sym_map.get(keycode, u"").lower()
    if control and meta: #equivalent to altgr so clear flags
        control = False
        meta = False    
    elif control:
        char = str(keycode)
    return KeyPress(char, shift, control, meta, keyname)




"""
Make sure that the re-enable notice is visible to the user on launch
"""
import sys
import subprocess

seash_process = subprocess.Popen([sys.executable, 'seash.py'], 
    stdin=subprocess.PIPE, 
    stdout=subprocess.PIPE, 
    stderr=subprocess.PIPE)

# Terminate seash...
seash_process.stdin.write('exit\n')
seash_process.stdin.close()

# Windows machines output \r\n, not \n for newlines.
# Replace them so we don't have to test for 2 cases.
stdout = seash_process.stdout.read().replace('\r\n', '\n')
stderr = seash_process.stderr.read().replace('\r\n', '\n')

expected_out = """Enabled modules: modules"""

# If we have any mismatched/unexpected output, print them to the relevant streams
if expected_out not in stdout:
  print stdout

if stderr:
  print>>sys.stderr, stderr


"""
Makes sure that vessels on the nodemanager are accessible before continuing with
the unit tests.  This is essential so that the first few unit tests don't fail 
because the node manager isn't ready yet.

Also disables all modules so that individual module tests can assume a clean
module system before running.
"""

# Needed so that the built-in type function is restored to the python
# abc library
import seash_dictionary

# Disable all modules.
# We pass in an empty dict because we only care that the .disabled file gets 
# created
import seash_modules
for module in seash_modules.get_enabled_modules():
  # The modules module must always be enabled.  Otherwise, we have no way of
  # enabling/disabling modules through the command loop.
  if module == 'modules':
    continue
  seash_modules.disable({}, module)

from repyportability import *
add_dy_support(locals())

dy_import_module_symbols('rsa.r2py')
dy_import_module_symbols('advertise.r2py')

for guestnum in xrange(4):
  guestkey = rsa_file_to_publickey('guest'+str(guestnum)+'.publickey')
  while not advertise_lookup(guestkey, graceperiod=1):
    sleep(2)



"""
Passes a series of commands to the parser in seash_dictionary and make sures
the parser returns the correct input dictionary.

Make sure to update this if the following command's dictionaries changes:
browse
show ownerinfo
set autosave
upload
"""
import seash_dictionary
import command_callbacks


resulting_dictionary = seash_dictionary.parse_command("browse")

expected_dictionary = {'browse':{'name':'browse', 'summary': 'Find vessels I can control', 'callback':command_callbacks.browse, 'help_text':"""
browse [advertisetype]

This command will use the default identity to search for vessels that can
be controlled.   Any vessel with the advertise flag set will be advertised
in at least one advertise service.   browse will look into these services
and add any vessels it can contact.

Setting advertisetype will restrict the advertise lookup to only use that 
service.   Some permitted values for advertisetype are central, DHT, and DOR.

Example:
exampleuser@ !> show targets
%all (empty)
exampleuser@ !> browse
['192.x.x.2:1224', '193.x.x.42:1224', '219.x.x.62:1224']
Added targets: %2(193.x.x.42:1224:v18), %3(219.x.x.62:1224:v4), %1(192.x.x.2:1224:v3)
Added group 'browsegood' with 3 targets
exampleuser@ !> show targets
browsegood ['192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%3 ['219.x.x.62:1224:v4']
%all ['192.x.x.2:1224:v3', '193.x.x.42:1224:v18', '219.x.x.62:1224:v4']
%1 ['192.x.x.2:1224:v3']
%2 ['193.x.x.42:1224:v18']

""", 'children':{}}}

if not resulting_dictionary == expected_dictionary:
  print "Incorrect values in command dictionary returned from parser: browse"




resulting_dictionary = seash_dictionary.parse_command("show ownerinfo")

expected_dictionary = {'show':{
    'name':'show', 
    'callback':command_callbacks.show, 
    'summary':"Displays the shell state (see 'help show')",
    'help_text':"""
Displays information regarding the current state of Seattle, depending on
the additional keywords that are passed in.

  (*) No need to update prior, the command contacts the nodes anew

""", 'children':{
    'ownerinfo':{
      'name':'ownerinfo', 
      'callback':command_callbacks.show_ownerinfo, 
      'summary': 'Display owner information for the vessels',
      'help_text':"""
show ownerinfo

This lists the ownerinfo strings for vessels in the default group.   See
'set ownerinfo' for more details
""", 'children':{}}}}}

if not resulting_dictionary == expected_dictionary:
  print "Incorrect values in command dictionary returned from parser: show ownerinfo"



resulting_dictionary = seash_dictionary.parse_command("set autosave off")

expected_dictionary = {'set':{'name':'set', 'callback':command_callbacks.set, 'summary': "Changes the shell or vessels (see 'help set')", 'help_text':"""
Changes the shell or vessels.
""", 'children':{
      'autosave':{
        'name':'autosave', 
        'callback':None, 
        'example':'[ on | off ]',
        'summary': "Sets whether to save the state after every command. Set to 'off' by default. The state is saved to a file called 'autosave_keyname', where keyname is the name of the current key you're using.",
        'help_text':"""
set autosave [on/off]

When turned on, the shell settings such as keys, targets, timeout value, etc.
will all be persisted to disk after every operation.   These are saved in a
file called 'autosave_(user's keyname)', which is encrypted with the default identity.   The user
can then restore the shell's state by typing 'loadstate identity'.

Example:
exampleuser@%1 !> set autosave on
exampleuser@%1 !> exit
(restart seash.py)
 !> loadkeys exampleuser
 !> as exampleuser
exampleuser@ !> loadstate autosave_exampleuser
exampleuser@%1 !>

""", 'children':{
          'off':{'name':'args', 'callback':command_callbacks.set_autosave_arg, 'help_text':'', 'children':{}},
      }},
  }}}

if not resulting_dictionary == expected_dictionary:
  print "Incorrect values in command dictionary returned from parser for command: set autosave off"



resulting_dictionary = seash_dictionary.parse_command("upload files.txt")

expected_dictionary = {
  'upload':{
    'name':'upload', 
    'callback':None, 
    'example':'localfn (remotefn)',
    'summary': 'Upload a file',
    'help_text':"""
upload srcfilename [destfilename]

Uploads a file into all vessels in the default group.   The file name that is
created in those vessels is destfilename (or srcfilename by default).

Example:
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': ''
exampleuser@%1 !> upload example.1.1.r2py
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': 'example.1.1.r2py'

""", 'children':{
      'files.txt':{'name':'filename', 'callback':command_callbacks.upload_filename, 'help_text':'', 'children':{}}}}}

if not resulting_dictionary == expected_dictionary:
  print "Incorrect values in command dictionary returned from parser: upload files.txt"




resulting_dictionary = seash_dictionary.parse_command("upload files.txt apples and oranges")

expected_dictionary = {
  'upload':{
    'name':'upload', 
    'callback':None, 
    'example': 'localfn (remotefn)',
    'summary': 'Upload a file',
    'help_text':"""
upload srcfilename [destfilename]

Uploads a file into all vessels in the default group.   The file name that is
created in those vessels is destfilename (or srcfilename by default).

Example:
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': ''
exampleuser@%1 !> upload example.1.1.r2py
exampleuser@%1 !> show files
Files on '192.x.x.2:1224:v3': 'example.1.1.r2py'

""", 'children':{
      'files.txt':{'name':'filename', 'callback':command_callbacks.upload_filename, 'help_text':'', 'children':{
          'apples and oranges':{'name':'args', 'callback':command_callbacks.upload_filename_remotefn, 'help_text':'', 'children':{}}
      }},
  }}}

if not resulting_dictionary == expected_dictionary:
  print "Incorrect values in command dictionary returned from parser: upload files.txt apples and oranges"



"""
Executes a few help commands and make sures seash prints out the correct output.
Make sure to update the txt file of the expected output if help's contents have
modified
"""
import seash
import sys

# We need to do module manipulation
import seash_dictionary
import seash_modules
#pragma out Enabled modules: modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)

orig_stdout = sys.stdout
sys.stdout = open("test_results.txt", "w")
command_list = [
  'help show',
  'enable geoip',
  'help show',
  'help show location',
  ]

seash.command_loop(command_list)



sys.stdout.close()

# Compares the test results with the expected text that should have been printed
# out when executing the series of help commands
sys.stdout = orig_stdout

test_results = open("test_results.txt", "r")
wanted_results = open("help_dynamic_test_results.txt", "r")

open('asdf.txt', 'w').write(test_results.read())
test_results = open("test_results.txt", "r")

original = wanted_results.readlines()
actual = test_results.readlines()

for i in range(len(original)):
  if not original[i] == actual[i]:
    print "Line " + str(i) + " of test results are not consistent with expected results: help_test_results.txt"



test_results.close()
wanted_results.close()

# Disable the modules we used
seash.command_loop(['disable geoip'])


"""
Author: Alan Loh

Module: A library to hold the definitions of common 
        seash exceptions and errors.

To avoid declaring or inheriting the same exception 
definitions from other files, declare all new exceptions
and errors here for easier accessibility by other files.
"""

# Base class for module-related errors/exceptions.
class ModuleError(Exception):
  pass

# There is a conflict between two modules.
# The string passed in should be the conflicting command.
class ModuleConflictError(ModuleError):
  pass

# Used by module system for errors in importing modules
class ModuleImportError(ModuleError):
  pass


# Used by command parser for errors in reading commands
class DispatchError(Exception):
  pass

# Used by the command parser for invalid command inputs
class ParseError(Exception):
  pass

# Use this to signal an error we want to print...
class UserError(Exception):
  """This indicates the user typed an incorrect command"""
  pass

# Use this to signal an error during initializing module.
class InitializeError(Exception):
  pass


"""
Test that the geoip module functions correctly.

"""
import os
import seash
import sys

# We need the modules module
import seash_modules
import seash_dictionary
#pragma out Enabled modules: modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)


commands = [
  'enable geoip',
  'disable geoip',
  'enable geoip',
  'loadkeys guest0',
  'as guest0',
  'browse',
  'on %1',
  'show location',
  'show coordinates'
]

# Allow the geoip module to be disabled when done, 
# even if exceptions occur
try:
  seash.command_loop(commands)
except Exception, e:
  print>>sys.stderr, str(e)

# Make sure the geoip module is disabled after this run.
# This is to ensure that other modules' tests are not affected by this module.
seash.command_loop(['disable geoip'])


# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006  Jorgen Stenarson. <jorgen.stenarson@bostream.nu>
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
import cPickle
import logging
import logging.handlers
import SocketServer
import struct,socket

try:
    import msvcrt
except ImportError:
    msvcrt = None
    print u"problem"


port = logging.handlers.DEFAULT_TCP_LOGGING_PORT
host = u'localhost'

def check_key():
    if msvcrt is None:
        return False
    else:
        if msvcrt.kbhit() != 0:
            q = msvcrt.getch()
            return q
    return u""


singleline=False

def main():
    print u"Starting TCP logserver on port:", port
    print u"Press q to quit logserver", port
    print u"Press c to clear screen", port
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    s.bind((u"", port))
    s.settimeout(1)
    while 1:
        try:
            data, addr = s.recvfrom(100000)
            print data,
        except socket.timeout:
            key = check_key().lower()
            if u"q" == key:
                print u"Quitting logserver"
                break
            elif u"c" == key:
                print u"\n" * 100            

if __name__ == u"__main__":
    main()


"""
Loads a series of keys and make sures seash recognizes them correctly.
"""
import seash
import sys
from repyportability import add_dy_support
add_dy_support(locals())
dy_import_module_symbols('rsa.r2py')


orig_stdout = sys.stdout

# Prepare the expected results.

# This list of tuples dictates whether the specified guest has its 
# public/private keys loaded.
# Each tuple represents: (guestname, publickey, privatekey)
guest_has_keys = [('guest2', True, True),
                  ('guest0', True, False),
                  ('guest1', False, True)]

sys.stdout = open("loadkeys_test_results.txt", "w")

# show keys
# Guestname Publickey/None Privatekey/None
# None is printed only if that key isn't loaded.
# e.g.:
# guest0 {'e': 65537L, 'n': 899L} {'q': 31L, 'p': 29L, 'd': 593L}
# guest1 None {'q': 31L, 'p': 29L, 'd': 593L}
# guest2 {'e': 65537L, 'n': 899L} None

for (guest, haspublic, hasprivate) in guest_has_keys:
  print guest,
  print rsa_file_to_publickey(guest + ".publickey") if haspublic else None,
  print rsa_file_to_privatekey(guest + ".privatekey") if hasprivate else None

# show identities
# Same as show keys, except:
#   Keys are replaced by the tokens "PUB" and "PRIV"
#   If a key is not found, it is simply not printed
for (guest, haspublic, hasprivate) in guest_has_keys:
  print guest, 
  if haspublic:
    print "PUB", 
  if hasprivate:
    print "PRIV",
  print

sys.stdout.close()

# Begin testing
sys.stdout = open("test_results.txt", "w")
command_list = [
  'loadpub guest0',
  'loadpriv guest1', 
  'loadkeys guest2', 
  'show keys', 
  'show identities'
  ]

seash.command_loop(command_list)

sys.stdout.close()


# Compare the results to make sure key values and identities as recognized
# by seash are identical.
sys.stdout = orig_stdout

test_results = open("test_results.txt", "r")
wanted_results = open("loadkeys_test_results.txt", "r")

original = wanted_results.readlines()
actual = test_results.readlines()

for i in range(len(original)):
  if not original[i].startswith(actual[i]):
    print "Line " + str(i) + " of test results are not consistent with expected results: loadkeys_test_results.txt"

test_results.close()
wanted_results.close()




"""
Test commands that manipulate the modules within seash

"""
import sys
import seash
import seash_dictionary
import seash_modules

#pragma out Enabled modules: modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)


oldstdout = sys.stdout
sys.stdout = open('test_results.txt', 'w')


seash.command_loop([
  'show modules',
  'enable geoip',
  'show modules',
  'modulehelp geoip',
  'disable geoip',
])

sys.stdout = oldstdout
expected_lines = open('modulemanipulation_test_results.txt', 'r').readlines()
actual_lines = open('test_results.txt', 'r').readlines()

for lineno in xrange(len(actual_lines)):
  if expected_lines[lineno] != actual_lines[lineno]:
    print "Line", lineno, "of test results are not consistent with expected results: modulemanipulation_test_results.txt"


"""
<Program Name>
  conflict/__init__.py

<Purpose>
  Module that is used with ut_seash_moduleconflicterror.py.  It should not be
  enabled successfully, as it will conflict with the default 'show' command.

  Other fields are simply defined so that the module importer can import the
  module without problems.
"""

moduledata = {
  'help_text': '',
  'url': None,
  'command_dict': {
    'show': { 'name':'nonexistant', 'help_text': 'bad help text',
              'callback': None, 'children': {}},
  }
}


"""
Make sure that preprocessed command line input is printed to the screen
"""

import sys

import seash_dictionary

#pragma out Enabled modules:
import seash

import seash_modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)

# There shouldn't be any output from here on...
# If the file has anything inside it after we're done, then it is an error.
sys.stdout = open('test_results.txt', 'w')

seash.command_loop([
  'enable variables',
  'set showparse off',
  'set username guest0',
  'loadkeys $username',
  'as $username$',
  'disable variables'])

sys.stdout.close()
sys.stdout = sys.__stdout__

test_output = open('test_results.txt', 'r').read()
if test_output:
  print test_output
  


"""
Makes sure the Variables module for seash works as intended.
In particular, we check that trailing delimiters are not recognized as a
variable.
"""
import seash
import seash_dictionary
import seash_exceptions
import seash_modules

#pragma out Enabled modules: modules
seash_modules.enable_modules_from_last_session(seash_dictionary.seashcommanddict)


# This error should come up since %all contains nothing as we don't browse for
# anything prior to this.  This is not a problem though as we are just testing
# the trailing $ in a command.
try:
  seash.command_loop([
    'enable variables',
    'add %all to $',
  ])
except seash_exceptions.UserError, e:
  if not "No targets to add (the target is already in '$'" in str(e):
    raise

seash.command_loop(['disable variables'])


"""
<Filename>
  variables/__init__.py

<Purpose>
  This is the main module file for the variables seash module.  It provides
  users with the ability to set custom variables and then retrieve their values
  for future use.

  It implements the following commands:
    set [variable name] [variable value]
    show variables
"""

# Needed so that we can relay user errors to seash for printing
import seash_exceptions


# Keeps track of user defined variables
# Maps a variable name (string) to a value (string)
uservariables = {}


help_text = """
Variables Module

This module allows you to define variables that you can access later using the
'set' command.  Variable names cannot contain spaces.  Values are automatically
stripped of leading/trailing whitespace.  To set a new variable, use the 
following syntax:

  user@! > set uploadfn theverylonguploadfile.txt

You reference the variable by typing '$' and then the variable name.
Optionally you can also mark the end of a variable by using yet another '$'.

  user@! > on %1 upload $uploadfn
  user@! > on %1 upload dir/$uploadfn
  user@! > on %1 upload dir/$username$/password

If you need to use the $ symbol, you can mark that by using 2 subsequent $'s.
For example, if you type the following:
  user@! > on %1 run need_dollarsign_arg.r2py $$ username

It is converted to:
  user@! > on %1 run need_dollarsign_arg.r2py $ username  

Escaping is optional for a single $ at the end of the string.  The following 
two commands are interpreted as identical commands.

  user@! > on %1 run need_dollarsign_arg.r2py $$
  user@! > on %1 run need_dollarsign_arg.r2py $

"""


def set_user_variable(input_dict, environment_dict):
  """
  <Purpose>
    Seash callback to allow user to define a custom variable and assign a value
    to it.
  <Arguments>
    input_dict: Input dictionary generated by seash_dictionary.parse_command().
    environment_dict: Dictionary describing the current seash environment.
    
    For more information, see command_callbacks.py's module docstring.
  <Side Effects>
    A new variable will be added to the uservariables dictionary.
  <Exceptions>
    UserError: The user did not provide a value to assign to the variable
  <Return>
    None
  """
  # Iterates through the dictionary to retrieve the variable name
  command_key = input_dict.keys()[0]
  while input_dict[command_key]['name'] is not 'variable_name':
    input_dict = input_dict[command_key]['children']
    variable_name = command_key = input_dict.keys()[0]

  # User may accidentally omit this value, we must catch the exception and
  # return something readable...
  try:
    # Iterates through the dictionary to retrieve the variable value
    while input_dict[command_key]['name'] is not 'variable_value':
      input_dict = input_dict[command_key]['children']
      variable_value = command_key = input_dict.keys()[0]
  except IndexError, e:
    raise seash_exceptions.UserError("Error, expected a value to assign to variable")

  uservariables[variable_name] = variable_value.strip()



def show_user_variables(input_dict, environment_dict):
  """
  <Purpose>
    Seash callback to allow user to check all variables that they defined.
  <Arguments>
    input_dict: Input dictionary generated by seash_dictionary.parse_command().
    environment_dict: Dictionary describing the current seash environment.
    
    For more information, see command_callbacks.py's module docstring.
  <Side Effects>
    All the variables currently defined will be printed alongside their values.
  <Exceptions>
    None
  <Return>
    None
  """
  for variable, value in uservariables.iteritems():
    print variable+": '"+value+"'"


def preprocess_user_variables(userinput):
  """
  <Purpose>
    Command parser for user variables.  Takes the raw userinput and replaces
    each user variable with its set value.
  <Arguments>
    userinput: A raw user string
  <Side Effects>
    Each user variable will be replaced by the value that it was previously
    set to.
  <Exceptions>
    UserError: User typed an unrecognized or invalid variable name
  <Returns>
    The preprocessed string
  """
  retstr = ""
  while '$' in userinput:
    text_before_variable , variable_delimiter, userinput = userinput.partition('$')
    retstr += text_before_variable

    # Treat $$ as an escape for a single $.
    # Also escape if there is nothing left
    if not userinput or userinput.startswith('$'):
      retstr += '$'
      userinput = userinput[1:]
      continue

    # Look for the next space, or the next $.  The closest one of these will be
    # used as the delimiter.  Then update the remaining user input.
    space_variable_length = userinput.find(' ')
    dollarsign_variable_length = userinput.find('$')

    # If the length is -1, then the delimiter was not found.
    # We use the length of the entire string to represent this.
    # If there was one delimiter found, then that delimiter's value
    # will always be less than the string's length.
    # If it is a tie, then it simply means that the entire string
    # is the variable name.
    if space_variable_length == -1:
      space_variable_length = len(userinput)
    if dollarsign_variable_length == -1:
      dollarsign_variable_length = len(userinput)

    variable_length = min(space_variable_length, dollarsign_variable_length)
    variable_name = userinput[:variable_length]
    userinput = userinput[variable_length + 1:]  # Skip the actual delimiter

    # Perform the replacement!
    # User may type in a variable that has not yet been defined
    try:
      retstr += uservariables[variable_name]
    except KeyError:
      raise seash_exceptions.UserError("Variable does not exist: "+variable_name)

    # The user expects a space before the string right after the variable.
    # e.g. 'loadkeys $myname as awesome' should turn into
    #      'loadkeys theusername as awesome'
    if space_variable_length < dollarsign_variable_length:
      retstr += ' '

  # Now add the remaining text after the last variable
  else:
    retstr += userinput

  return retstr


def autocomplete(input_list):
  """
  <Purpose>
    Returns all valid input completions for the specified command line input.

  <Arguments>
    input_list: A list of tokens.

  <Side Effects>
    None

  <Exceptions>
    None

  <Returns>
    A list of strings representing valid completions.
  """
  # We are only interested if the last token starts with a single '$'
  # Double $$'s indicate that it is meant to be a '$', so we don't do anything.
  if input_list[-1].startswith('$') and not input_list[-1].startswith('$$'):
    # Omit the '$'
    partial_variable = input_list[-1][1:]
    commands = []

    for variable in uservariables:
      # No need to include variables that don't match...
      if variable.startswith(partial_variable):
        # Reconstruct the string
        tokens = input_list[:-1] + ['$'+variable]
        commands.append(' '.join(tokens))
    return commands
  return []



command_dict = {
  'set': {'children': {
    '[ARGUMENT]': {'name':'variable_name', 'callback':set_user_variable, 'help_text':'', 'summary': 'Sets a custom user variable.', 'children':{
      '[ARGUMENT]': {'name':'variable_value', 'callback':None, 'help_text':'', 'children':{}}
    }},
  }},
  'show': {'children': {
    'variables': {'name': 'variables', 'callback':show_user_variables, 'help_text':'', 'summary':"Display a list of user defined variables.", 'children': {}}
  }}
}


# This is where the module importer loads the module from
moduledata = {
  'command_dict': command_dict,
  'help_text': help_text,
  'url': None,
  'input_preprocessor': preprocess_user_variables,
  'tab_completer': autocomplete
}
